# Copyright 2019 Gentoo Authors
# Distributed under the terms of the GNU General Public License v2

from portage.tests import TestCase
from portage.tests.resolver.ResolverPlayground import (ResolverPlayground,
	ResolverPlaygroundTestCase)

class SlotConflictUpdateVirtTestCase(TestCase):

	def testSlotConflictUpdateVirt(self):

		ebuilds = {
			"dev-db/mysql-connector-c-6.1.11-r2" : {
				"EAPI": "7",
				"SLOT" : "0/18"
			},

			"dev-db/mysql-connector-c-8.0.17-r3" : {
				"EAPI": "7",
				"SLOT" : "0/21"
			},

			"virtual/libmysqlclient-18-r1" : {
				"EAPI": "7",
				"SLOT" : "0/18",
				"RDEPEND": "dev-db/mysql-connector-c:0/18",
			},

			"virtual/libmysqlclient-21" : {
				"EAPI": "7",
				"SLOT" : "0/21",
				"RDEPEND": "dev-db/mysql-connector-c:0/21",
			},

			"dev-perl/DBD-mysql-4.44.0" : {
				"EAPI": "7",
				"RDEPEND": "virtual/libmysqlclient:=",
			},
		}

		installed = {
			"dev-db/mysql-connector-c-6.1.11-r2" : {
				"EAPI": "7",
				"SLOT" : "0/18"
			},

			"virtual/libmysqlclient-18-r1" : {
				"EAPI": "7",
				"SLOT" : "0/18",
				"RDEPEND": "dev-db/mysql-connector-c:0/18",
			},

			"dev-perl/DBD-mysql-4.44.0" : {
				"EAPI": "7",
				"RDEPEND": "virtual/libmysqlclient:0/18=",
			},
		}

		world = ["dev-db/mysql-connector-c", "dev-perl/DBD-mysql"]

		test_cases = (
			# In order to avoid missed updates for bug 692746, consider
			# masking a package matched by all parent atoms.
			ResolverPlaygroundTestCase(
				['@world'],
				options = {"--update": True, "--deep": True},
				success = True,
				mergelist = ['dev-db/mysql-connector-c-8.0.17-r3', 'virtual/libmysqlclient-21', 'dev-perl/DBD-mysql-4.44.0']),
		)

		playground = ResolverPlayground(ebuilds=ebuilds,
			installed=installed, world=world, debug=False)
		try:
			for test_case in test_cases:
				playground.run_TestCase(test_case)
				self.assertEqual(test_case.test_success, True, test_case.fail_msg)
		finally:
			playground.debug = False
			playground.cleanup()



# Copyright 2010-2011 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

import re
import portage
from portage import os
from portage.const import PORTAGE_PYM_PATH
from portage.tests import TestCase
from portage.util._eventloop.global_event_loop import global_event_loop

from _emerge.PipeReader import PipeReader
from _emerge.SpawnProcess import SpawnProcess

class LazyImportPortageBaselineTestCase(TestCase):

	_module_re = re.compile(r'^(portage|repoman|_emerge)\.')

	_baseline_imports = frozenset([
		'portage.const', 'portage.localization',
		'portage.proxy', 'portage.proxy.lazyimport',
		'portage.proxy.objectproxy',
		'portage._selinux',
	])

	_baseline_import_cmd = [portage._python_interpreter, '-c', '''
import os
import sys
sys.path.insert(0, os.environ["PORTAGE_PYM_PATH"])
import portage
sys.stdout.write(" ".join(k for k in sys.modules
	if sys.modules[k] is not None))
''']

	def testLazyImportPortageBaseline(self):
		"""
		Check what modules are imported by a baseline module import.
		"""

		env = os.environ.copy()
		pythonpath = env.get('PYTHONPATH')
		if pythonpath is not None and not pythonpath.strip():
			pythonpath = None
		if pythonpath is None:
			pythonpath = ''
		else:
			pythonpath = ':' + pythonpath
		pythonpath = PORTAGE_PYM_PATH + pythonpath
		env['PYTHONPATH'] = pythonpath

		# If python is patched to insert the path of the
		# currently installed portage module into sys.path,
		# then the above PYTHONPATH override doesn't help.
		env['PORTAGE_PYM_PATH'] = PORTAGE_PYM_PATH

		scheduler = global_event_loop()
		master_fd, slave_fd = os.pipe()
		master_file = os.fdopen(master_fd, 'rb', 0)
		slave_file = os.fdopen(slave_fd, 'wb')
		producer = SpawnProcess(
			args=self._baseline_import_cmd,
			env=env, fd_pipes={1:slave_fd},
			scheduler=scheduler)
		producer.start()
		slave_file.close()

		consumer = PipeReader(
			input_files={"producer" : master_file},
			scheduler=scheduler)

		consumer.start()
		consumer.wait()
		self.assertEqual(producer.wait(), os.EX_OK)
		self.assertEqual(consumer.wait(), os.EX_OK)

		output = consumer.getvalue().decode('ascii', 'replace').split()

		unexpected_modules = " ".join(sorted(x for x in output \
			if self._module_re.match(x) is not None and \
			x not in self._baseline_imports))

		self.assertEqual("", unexpected_modules)



# Copyright 2012 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

from portage import os, _supported_eapis
from portage.dep import use_reduce
from portage.eapi import eapi_has_automatic_unpack_dependencies
from portage.exception import InvalidDependString
from portage.localization import _
from portage.util import grabfile, writemsg

def load_unpack_dependencies_configuration(repositories):
	repo_dict = {}
	for repo in repositories.repos_with_profiles():
		for eapi in _supported_eapis:
			if eapi_has_automatic_unpack_dependencies(eapi):
				file_name = os.path.join(repo.location, "profiles", "unpack_dependencies", eapi)
				lines = grabfile(file_name, recursive=True)
				for line in lines:
					elements = line.split()
					suffix = elements[0].lower()
					if len(elements) == 1:
						writemsg(_("--- Missing unpack dependencies for '%s' suffix in '%s'\n") % (suffix, file_name))
					depend = " ".join(elements[1:])
					try:
						use_reduce(depend, eapi=eapi)
					except InvalidDependString as e:
						writemsg(_("--- Invalid unpack dependencies for '%s' suffix in '%s': '%s'\n" % (suffix, file_name, e)))
					else:
						repo_dict.setdefault(repo.name, {}).setdefault(eapi, {})[suffix] = depend

	ret = {}
	for repo in repositories.repos_with_profiles():
		for repo_name in [x.name for x in repo.masters] + [repo.name]:
			for eapi in repo_dict.get(repo_name, {}):
				for suffix, depend in repo_dict.get(repo_name, {}).get(eapi, {}).items():
					ret.setdefault(repo.name, {}).setdefault(eapi, {})[suffix] = depend

	return ret



# Copyright 2018 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

from portage.tests import TestCase
from portage.util.install_mask import InstallMask


class InstallMaskTestCase(TestCase):

	def testTrailingSlash(self):
		"""
		Test that elements with a trailing slash match a directory
		but not a regular file.
		"""
		cases = (
			(
				'/foo/bar/ -/foo/bar/*.foo -*.baz',
				(
					(
						'foo/bar/baz',
						True,
					),
					(
						'foo/bar/',
						True,
					),
					# /foo/bar/ does not match
					(
						'foo/bar',
						False,
					),
					# this is excluded
					(
						'foo/bar/baz.foo',
						False,
					),
					# this is excluded
					(
						'foo/bar/baz.baz',
						False,
					),
					(
						'foo/bar/baz.bar',
						True,
					),
				)
			),
			(
				'/foo/bar -/foo/bar/*.foo -*.baz',
				(
					(
						'foo/bar/baz',
						True,
					),
					# /foo/bar matches both foo/bar/ and foo/bar
					(
						'foo/bar/',
						True,
					),
					(
						'foo/bar',
						True,
					),
					# this is excluded
					(
						'foo/bar/baz.foo',
						False,
					),
					# this is excluded
					(
						'foo/bar/baz.baz',
						False,
					),
					(
						'foo/bar/baz.bar',
						True,
					),
				)
			),
			(
				'/foo*',
				(
					(
						'foo',
						True,
					),
					(
						'foo/',
						True,
					),
					(
						'foobar',
						True,
					),
					(
						'foobar/',
						True,
					),
				)
			),
			(
				'/foo*/',
				(
					(
						'foo',
						False,
					),
					(
						'foo/',
						True,
					),
					(
						'foobar',
						False,
					),
					(
						'foobar/',
						True,
					),
				)
			),
			(
				'/usr/share/locale '
				'-/usr/share/locale/en* '
				'-/usr/share/locale/kf5_all_languages '
				'-/usr/share/locale/locale.alias',
				(
					(
						'usr/share/locale/en',
						False,
					),
					(
						'usr/share/locale/en_GB',
						False,
					),
					(
						'usr/share/locale/en/kf5_all_languages',
						False,
					),
					(
						'usr/share/locale/locale.alias',
						False,
					),
					(
						'usr/share/locale/es',
						True,
					),
					(
						'usr/share/locale/fr',
						True,
					),
					(
						'usr/share/locale',
						True,
					),
				)
			),
		)

		for install_mask_str, paths in cases:
			install_mask = InstallMask(install_mask_str)
			for path, expected in paths:
				self.assertEqual(install_mask.match(path), expected,
					'unexpected match result for "{}" with path {}'.\
					format(install_mask_str, path))



# Copyright 2010-2020 Gentoo Authors
# Distributed under the terms of the GNU General Public License v2


from portage import _encodings, _unicode_encode
from portage.exception import PortageException
from portage.tests import TestCase
from _emerge.DependencyArg import DependencyArg
from _emerge.UseFlagDisplay import UseFlagDisplay


class StringFormatTestCase(TestCase):
	"""
	Test that string formatting works correctly in the current interpretter,
	which may be either python2 or python3.
	"""

	unicode_strings = (
		'\u2018',
		'\u2019',
	)

	def testDependencyArg(self):

		self.assertEqual(_encodings['content'], 'utf_8')

		for arg_unicode in self.unicode_strings:
			arg_bytes = _unicode_encode(arg_unicode, encoding=_encodings['content'])
			dependency_arg = DependencyArg(arg=arg_unicode)

			formatted_str = "%s" % (dependency_arg,)
			self.assertEqual(formatted_str, arg_unicode)

			# Test the __str__ method which returns unicode in python3
			formatted_str = "%s" % (dependency_arg,)
			self.assertEqual(formatted_str, arg_unicode)

	def testPortageException(self):

		self.assertEqual(_encodings['content'], 'utf_8')

		for arg_unicode in self.unicode_strings:
			arg_bytes = _unicode_encode(arg_unicode, encoding=_encodings['content'])
			e = PortageException(arg_unicode)

			formatted_str = "%s" % (e,)
			self.assertEqual(formatted_str, arg_unicode)

			# Test the __str__ method which returns unicode in python3
			formatted_str = "%s" % (e,)
			self.assertEqual(formatted_str, arg_unicode)

	def testUseFlagDisplay(self):

		self.assertEqual(_encodings['content'], 'utf_8')

		for enabled in (True, False):
			for forced in (True, False):
				for arg_unicode in self.unicode_strings:
					e = UseFlagDisplay(arg_unicode, enabled, forced)

					formatted_str = "%s" % (e,)
					self.assertEqual(isinstance(formatted_str, str), True)

					# Test the __str__ method which returns unicode in python3
					formatted_str = "%s" % (e,)
					self.assertEqual(isinstance(formatted_str, str), True)



# Copyright 2010-2020 Gentoo Authors
# Distributed under the terms of the GNU General Public License v2

__all__ = ['env_update']

import errno
import glob
import io
import stat
import time

import portage
from portage import os, _encodings, _unicode_decode, _unicode_encode
from portage.checksum import prelink_capable
from portage.data import ostype
from portage.exception import ParseError
from portage.localization import _
from portage.process import find_binary
from portage.util import atomic_ofstream, ensure_dirs, getconfig, \
	normalize_path, writemsg
from portage.util.listdir import listdir
from portage.dbapi.vartree import vartree
from portage.package.ebuild.config import config


def env_update(makelinks=1, target_root=None, prev_mtimes=None, contents=None,
	env=None, writemsg_level=None, vardbapi=None):
	"""
	Parse /etc/env.d and use it to generate /etc/profile.env, csh.env,
	ld.so.conf, and prelink.conf. Finally, run ldconfig. When ldconfig is
	called, its -X option will be used in order to avoid potential
	interference with installed soname symlinks that are required for
	correct operation of FEATURES=preserve-libs for downgrade operations.
	It's not necessary for ldconfig to create soname symlinks, since
	portage will use NEEDED.ELF.2 data to automatically create them
	after src_install if they happen to be missing.
	@param makelinks: True if ldconfig should be called, False otherwise
	@param target_root: root that is passed to the ldconfig -r option,
		defaults to portage.settings["ROOT"].
	@type target_root: String (Path)
	"""
	if vardbapi is None:
		if isinstance(env, config):
			vardbapi = vartree(settings=env).dbapi
		else:
			if target_root is None:
				eprefix = portage.settings["EPREFIX"]
				target_root = portage.settings["ROOT"]
				target_eroot = portage.settings['EROOT']
			else:
				eprefix = portage.const.EPREFIX
				target_eroot = os.path.join(target_root,
					eprefix.lstrip(os.sep))
				target_eroot = target_eroot.rstrip(os.sep) + os.sep
			if hasattr(portage, "db") and target_eroot in portage.db:
				vardbapi = portage.db[target_eroot]["vartree"].dbapi
			else:
				settings = config(config_root=target_root,
					target_root=target_root, eprefix=eprefix)
				target_root = settings["ROOT"]
				if env is None:
					env = settings
				vardbapi = vartree(settings=settings).dbapi

	# Lock the config memory file to prevent symlink creation
	# in merge_contents from overlapping with env-update.
	vardbapi._fs_lock()
	try:
		return _env_update(makelinks, target_root, prev_mtimes, contents,
			env, writemsg_level)
	finally:
		vardbapi._fs_unlock()

def _env_update(makelinks, target_root, prev_mtimes, contents, env,
	writemsg_level):
	if writemsg_level is None:
		writemsg_level = portage.util.writemsg_level
	if target_root is None:
		target_root = portage.settings["ROOT"]
	if prev_mtimes is None:
		prev_mtimes = portage.mtimedb["ldpath"]
	if env is None:
		settings = portage.settings
	else:
		settings = env

	eprefix = settings.get("EPREFIX", "")
	eprefix_lstrip = eprefix.lstrip(os.sep)
	eroot = normalize_path(os.path.join(target_root, eprefix_lstrip)).rstrip(os.sep) + os.sep
	envd_dir = os.path.join(eroot, "etc", "env.d")
	ensure_dirs(envd_dir, mode=0o755)
	fns = listdir(envd_dir, EmptyOnError=1)
	fns.sort()
	templist = []
	for x in fns:
		if len(x) < 3:
			continue
		if not x[0].isdigit() or not x[1].isdigit():
			continue
		if x.startswith(".") or x.endswith("~") or x.endswith(".bak"):
			continue
		templist.append(x)
	fns = templist
	del templist

	space_separated = set(["CONFIG_PROTECT", "CONFIG_PROTECT_MASK"])
	colon_separated = set(["ADA_INCLUDE_PATH", "ADA_OBJECTS_PATH",
		"CLASSPATH", "INFODIR", "INFOPATH", "KDEDIRS", "LDPATH", "MANPATH",
		  "PATH", "PKG_CONFIG_PATH", "PRELINK_PATH", "PRELINK_PATH_MASK",
		  "PYTHONPATH", "ROOTPATH"])

	config_list = []

	for x in fns:
		file_path = os.path.join(envd_dir, x)
		try:
			myconfig = getconfig(file_path, expand=False)
		except ParseError as e:
			writemsg("!!! '%s'\n" % str(e), noiselevel=-1)
			del e
			continue
		if myconfig is None:
			# broken symlink or file removed by a concurrent process
			writemsg("!!! File Not Found: '%s'\n" % file_path, noiselevel=-1)
			continue

		config_list.append(myconfig)
		if "SPACE_SEPARATED" in myconfig:
			space_separated.update(myconfig["SPACE_SEPARATED"].split())
			del myconfig["SPACE_SEPARATED"]
		if "COLON_SEPARATED" in myconfig:
			colon_separated.update(myconfig["COLON_SEPARATED"].split())
			del myconfig["COLON_SEPARATED"]

	env = {}
	specials = {}
	for var in space_separated:
		mylist = []
		for myconfig in config_list:
			if var in myconfig:
				for item in myconfig[var].split():
					if item and not item in mylist:
						mylist.append(item)
				del myconfig[var] # prepare for env.update(myconfig)
		if mylist:
			env[var] = " ".join(mylist)
		specials[var] = mylist

	for var in colon_separated:
		mylist = []
		for myconfig in config_list:
			if var in myconfig:
				for item in myconfig[var].split(":"):
					if item and not item in mylist:
						mylist.append(item)
				del myconfig[var] # prepare for env.update(myconfig)
		if mylist:
			env[var] = ":".join(mylist)
		specials[var] = mylist

	for myconfig in config_list:
		"""Cumulative variables have already been deleted from myconfig so that
		they won't be overwritten by this dict.update call."""
		env.update(myconfig)

	ldsoconf_path = os.path.join(eroot, "etc", "ld.so.conf")
	try:
		myld = io.open(_unicode_encode(ldsoconf_path,
			encoding=_encodings['fs'], errors='strict'),
			mode='r', encoding=_encodings['content'], errors='replace')
		myldlines = myld.readlines()
		myld.close()
		oldld = []
		for x in myldlines:
			#each line has at least one char (a newline)
			if x[:1] == "#":
				continue
			oldld.append(x[:-1])
	except (IOError, OSError) as e:
		if e.errno != errno.ENOENT:
			raise
		oldld = None

	newld = specials["LDPATH"]
	if oldld != newld:
		#ld.so.conf needs updating and ldconfig needs to be run
		myfd = atomic_ofstream(ldsoconf_path)
		myfd.write("# ld.so.conf autogenerated by env-update; make all changes to\n")
		myfd.write("# contents of /etc/env.d directory\n")
		for x in specials["LDPATH"]:
			myfd.write(x + "\n")
		myfd.close()

	potential_lib_dirs = set()
	for lib_dir_glob in ('usr/lib*', 'lib*'):
		x = os.path.join(eroot, lib_dir_glob)
		for y in glob.glob(_unicode_encode(x,
			encoding=_encodings['fs'], errors='strict')):
			try:
				y = _unicode_decode(y,
					encoding=_encodings['fs'], errors='strict')
			except UnicodeDecodeError:
				continue
			if os.path.basename(y) != 'libexec':
				potential_lib_dirs.add(y[len(eroot):])

	# Update prelink.conf if we are prelink-enabled
	if prelink_capable:
		prelink_d = os.path.join(eroot, 'etc', 'prelink.conf.d')
		ensure_dirs(prelink_d)
		newprelink = atomic_ofstream(os.path.join(prelink_d, 'portage.conf'))
		newprelink.write("# prelink.conf autogenerated by env-update; make all changes to\n")
		newprelink.write("# contents of /etc/env.d directory\n")

		for x in sorted(potential_lib_dirs) + ['bin', 'sbin']:
			newprelink.write('-l /%s\n' % (x,))
		prelink_paths = set()
		prelink_paths |= set(specials.get('LDPATH', []))
		prelink_paths |= set(specials.get('PATH', []))
		prelink_paths |= set(specials.get('PRELINK_PATH', []))
		prelink_path_mask = specials.get('PRELINK_PATH_MASK', [])
		for x in prelink_paths:
			if not x:
				continue
			if x[-1:] != '/':
				x += "/"
			plmasked = 0
			for y in prelink_path_mask:
				if not y:
					continue
				if y[-1] != '/':
					y += "/"
				if y == x[0:len(y)]:
					plmasked = 1
					break
			if not plmasked:
				newprelink.write("-h %s\n" % (x,))
		for x in prelink_path_mask:
			newprelink.write("-b %s\n" % (x,))
		newprelink.close()

		# Migration code path.  If /etc/prelink.conf was generated by us, then
		# point it to the new stuff until the prelink package re-installs.
		prelink_conf = os.path.join(eroot, 'etc', 'prelink.conf')
		try:
			with open(_unicode_encode(prelink_conf,
				encoding=_encodings['fs'], errors='strict'), 'rb') as f:
				if f.readline() == b'# prelink.conf autogenerated by env-update; make all changes to\n':
					f = atomic_ofstream(prelink_conf)
					f.write('-c /etc/prelink.conf.d/*.conf\n')
					f.close()
		except IOError as e:
			if e.errno != errno.ENOENT:
				raise

	current_time = int(time.time())
	mtime_changed = False

	lib_dirs = set()
	for lib_dir in set(specials['LDPATH']) | potential_lib_dirs:
		x = os.path.join(eroot, lib_dir.lstrip(os.sep))
		try:
			newldpathtime = os.stat(x)[stat.ST_MTIME]
			lib_dirs.add(normalize_path(x))
		except OSError as oe:
			if oe.errno == errno.ENOENT:
				try:
					del prev_mtimes[x]
				except KeyError:
					pass
				# ignore this path because it doesn't exist
				continue
			raise
		if newldpathtime == current_time:
			# Reset mtime to avoid the potential ambiguity of times that
			# differ by less than 1 second.
			newldpathtime -= 1
			os.utime(x, (newldpathtime, newldpathtime))
			prev_mtimes[x] = newldpathtime
			mtime_changed = True
		elif x in prev_mtimes:
			if prev_mtimes[x] == newldpathtime:
				pass
			else:
				prev_mtimes[x] = newldpathtime
				mtime_changed = True
		else:
			prev_mtimes[x] = newldpathtime
			mtime_changed = True

	if makelinks and \
		not mtime_changed and \
		contents is not None:
		libdir_contents_changed = False
		for mypath, mydata in contents.items():
			if mydata[0] not in ("obj", "sym"):
				continue
			head, tail = os.path.split(mypath)
			if head in lib_dirs:
				libdir_contents_changed = True
				break
		if not libdir_contents_changed:
			makelinks = False

	if "CHOST" in settings and "CBUILD" in settings and \
		settings["CHOST"] != settings["CBUILD"]:
		ldconfig = find_binary("%s-ldconfig" % settings["CHOST"])
	else:
		ldconfig = os.path.join(eroot, "sbin", "ldconfig")

	if ldconfig is None:
		pass
	elif not (os.access(ldconfig, os.X_OK) and os.path.isfile(ldconfig)):
		ldconfig = None

	# Only run ldconfig as needed
	if makelinks and ldconfig:
		# ldconfig has very different behaviour between FreeBSD and Linux
		if ostype == "Linux" or ostype.lower().endswith("gnu"):
			# We can't update links if we haven't cleaned other versions first, as
			# an older package installed ON TOP of a newer version will cause ldconfig
			# to overwrite the symlinks we just made. -X means no links. After 'clean'
			# we can safely create links.
			writemsg_level(_(">>> Regenerating %setc/ld.so.cache...\n") % \
				(target_root,))
			os.system("cd / ; %s -X -r '%s'" % (ldconfig, target_root))
		elif ostype in ("FreeBSD", "DragonFly"):
			writemsg_level(_(">>> Regenerating %svar/run/ld-elf.so.hints...\n") % \
				target_root)
			os.system(("cd / ; %s -elf -i " + \
				"-f '%svar/run/ld-elf.so.hints' '%setc/ld.so.conf'") % \
				(ldconfig, target_root, target_root))

	del specials["LDPATH"]

	notice      = "# THIS FILE IS AUTOMATICALLY GENERATED BY env-update.\n"
	notice     += "# DO NOT EDIT THIS FILE."
	penvnotice  = notice + " CHANGES TO STARTUP PROFILES\n"
	cenvnotice  = penvnotice[:]
	penvnotice += "# GO INTO /etc/profile NOT /etc/profile.env\n\n"
	cenvnotice += "# GO INTO /etc/csh.cshrc NOT /etc/csh.env\n\n"

	#create /etc/profile.env for bash support
	profile_env_path = os.path.join(eroot, "etc", "profile.env")
	with atomic_ofstream(profile_env_path) as outfile:
		outfile.write(penvnotice)

		env_keys = [x for x in env if x != "LDPATH"]
		env_keys.sort()
		for k in env_keys:
			v = env[k]
			if v.startswith('$') and not v.startswith('${'):
				outfile.write("export %s=$'%s'\n" % (k, v[1:]))
			else:
				outfile.write("export %s='%s'\n" % (k, v))

	# Create the systemd user environment configuration file
	# /etc/environment.d/10-gentoo-env.conf with the
	# environment configuration from /etc/env.d.
	systemd_environment_dir = os.path.join(eroot, "etc", "environment.d")
	os.makedirs(systemd_environment_dir, exist_ok=True)

	systemd_gentoo_env_path = os.path.join(systemd_environment_dir,
						"10-gentoo-env.conf")
	with atomic_ofstream(systemd_gentoo_env_path) as systemd_gentoo_env:
		senvnotice = notice + "\n\n"
		systemd_gentoo_env.write(senvnotice)

		for env_key in env_keys:
			# Skip PATH since this makes it impossible to use
			# "systemctl --user import-environment PATH".
			if env_key == 'PATH':
				continue

			env_key_value = env[env_key]

			# Skip variables with the empty string
			# as value. Those sometimes appear in
			# profile.env (e.g. "export GCC_SPECS=''"),
			# but are invalid in systemd's syntax.
			if not env_key_value:
				continue

			# Transform into systemd environment.d
			# conf syntax, basically shell variable
			# assignment (without "export ").
			line = f"{env_key}={env_key_value}\n"

			systemd_gentoo_env.write(line)

	#create /etc/csh.env for (t)csh support
	outfile = atomic_ofstream(os.path.join(eroot, "etc", "csh.env"))
	outfile.write(cenvnotice)
	for x in env_keys:
		outfile.write("setenv %s '%s'\n" % (x, env[x]))
	outfile.close()



# Copyright 2015 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

from portage.tests import TestCase
from portage.tests.resolver.ResolverPlayground import (ResolverPlayground,
	ResolverPlaygroundTestCase)

class SonameOrChoicesTestCase(TestCase):

	def testSonameConflictMissedUpdate(self):

		binpkgs = {
			"dev-lang/ocaml-4.02.1" : {
				"EAPI": "5",
				"PROVIDES": "x86_32: libocaml-4.02.1.so",
			},

			"dev-lang/ocaml-4.01.0" : {
				"EAPI": "5",
				"PROVIDES": "x86_32: libocaml-4.01.0.so",
			},

			"dev-ml/lablgl-1.05" : {
				"DEPEND": (">=dev-lang/ocaml-3.10.2 "
					"|| ( dev-ml/labltk <dev-lang/ocaml-4.02 )"),
				"RDEPEND": (">=dev-lang/ocaml-3.10.2 "
					"|| ( dev-ml/labltk <dev-lang/ocaml-4.02 )"),
				"REQUIRES": "x86_32: libocaml-4.02.1.so",
			},

			"dev-ml/labltk-8.06.0" : {
				"EAPI": "5",
				"SLOT": "0/8.06.0",
				"DEPEND": ">=dev-lang/ocaml-4.02",
				"RDEPEND": ">=dev-lang/ocaml-4.02",
				"REQUIRES": "x86_32: libocaml-4.02.1.so",
			},
		}

		installed = {
			"dev-lang/ocaml-4.01.0" : {
				"EAPI": "5",
				"PROVIDES": "x86_32: libocaml-4.01.0.so",
			},

			"dev-ml/lablgl-1.05" : {
				"DEPEND": (">=dev-lang/ocaml-3.10.2 "
					"|| ( dev-ml/labltk <dev-lang/ocaml-4.02 )"),
				"RDEPEND": (">=dev-lang/ocaml-3.10.2 "
					"|| ( dev-ml/labltk <dev-lang/ocaml-4.02 )"),
				"REQUIRES": "x86_32: libocaml-4.01.0.so",
			},
		}

		world = (
			"dev-lang/ocaml",
			"dev-ml/lablgl",
		)

		test_cases = (

			# bug #531656: If an ocaml update is desirable,
			# then we need to pull in dev-ml/labltk.
			ResolverPlaygroundTestCase(
				["@world"],
				options = {
					"--deep": True,
					"--ignore-soname-deps": "n",
					"--update": True,
					"--usepkgonly": True
				},
				success = True,
				mergelist = [
					"[binary]dev-lang/ocaml-4.02.1",
					"[binary]dev-ml/labltk-8.06.0",
					"[binary]dev-ml/lablgl-1.05",
				]
			),

		)

		playground = ResolverPlayground(debug=False,
			binpkgs=binpkgs, installed=installed, world=world)
		try:
			for test_case in test_cases:
				playground.run_TestCase(test_case)
				self.assertEqual(test_case.test_success, True,
					test_case.fail_msg)
		finally:
			# Disable debug so that cleanup works.
			playground.debug = False
			playground.cleanup()



# Copyright 2008-2020 Gentoo Authors
# Distributed under the terms of the GNU General Public License v2


__all__ = ['ObjectProxy']

class ObjectProxy:

	"""
	Object that acts as a proxy to another object, forwarding
	attribute accesses and method calls. This can be useful
	for implementing lazy initialization.
	"""

	__slots__ = ()

	def _get_target(self):
		raise NotImplementedError(self)

	def __getattribute__(self, attr):
		result = object.__getattribute__(self, '_get_target')()
		return getattr(result, attr)

	def __setattr__(self, attr, value):
		result = object.__getattribute__(self, '_get_target')()
		setattr(result, attr, value)

	def __call__(self, *args, **kwargs):
		result = object.__getattribute__(self, '_get_target')()
		return result(*args, **kwargs)

	def __enter__(self):
		return object.__getattribute__(self, '_get_target')().__enter__()

	def __exit__(self, exc_type, exc_value, traceback):
		return object.__getattribute__(self, '_get_target')().__exit__(
			exc_type, exc_value, traceback)

	def __setitem__(self, key, value):
		object.__getattribute__(self, '_get_target')()[key] = value

	def __getitem__(self, key):
		return object.__getattribute__(self, '_get_target')()[key]

	def __delitem__(self, key):
		del object.__getattribute__(self, '_get_target')()[key]

	def __contains__(self, key):
		return key in object.__getattribute__(self, '_get_target')()

	def __iter__(self):
		return iter(object.__getattribute__(self, '_get_target')())

	def __len__(self):
		return len(object.__getattribute__(self, '_get_target')())

	def __repr__(self):
		return repr(object.__getattribute__(self, '_get_target')())

	def __str__(self):
		return str(object.__getattribute__(self, '_get_target')())

	def __add__(self, other):
		return self.__str__() + other

	def __hash__(self):
		return hash(object.__getattribute__(self, '_get_target')())

	def __ge__(self, other):
		return object.__getattribute__(self, '_get_target')() >= other

	def __gt__(self, other):
		return object.__getattribute__(self, '_get_target')() > other

	def __le__(self, other):
		return object.__getattribute__(self, '_get_target')() <= other

	def __lt__(self, other):
		return object.__getattribute__(self, '_get_target')() < other

	def __eq__(self, other):
		return object.__getattribute__(self, '_get_target')() == other

	def __ne__(self, other):
		return object.__getattribute__(self, '_get_target')() != other

	def __bool__(self):
		return bool(object.__getattribute__(self, '_get_target')())

	def __int__(self):
		return int(object.__getattribute__(self, '_get_target')())



# Copyright 1998-2020 Gentoo Authors
# Distributed under the terms of the GNU General Public License v2

import logging
import operator
import sys
import signal

from _emerge.ProgressHandler import ProgressHandler

import portage
from portage import eapi_is_supported
from portage import os
from portage.cache.cache_errors import CacheError
from portage.eclass_cache import hashed_path
from portage.util import writemsg_level

def action_metadata(settings, portdb, myopts, porttrees=None):
	if porttrees is None:
		porttrees = portdb.porttrees
	portage.writemsg_stdout("\n>>> Updating Portage cache\n")
	cachedir = os.path.normpath(settings.depcachedir)
	if cachedir in ["/",    "/bin", "/dev",  "/etc",  "/home",
					"/lib", "/opt", "/proc", "/root", "/sbin",
					"/sys", "/tmp", "/usr",  "/var"]:
		print("!!! PORTAGE_DEPCACHEDIR IS SET TO A PRIMARY " + \
			"ROOT DIRECTORY ON YOUR SYSTEM.", file=sys.stderr)
		print("!!! This is ALMOST CERTAINLY NOT what you want: '%s'" % cachedir, file=sys.stderr)
		sys.exit(73)
	if not os.path.exists(cachedir):
		os.makedirs(cachedir)

	auxdbkeys = portdb._known_keys

	class TreeData:
		__slots__ = ('dest_db', 'eclass_db', 'path', 'src_db', 'valid_nodes')
		def __init__(self, dest_db, eclass_db, path, src_db):
			self.dest_db = dest_db
			self.eclass_db = eclass_db
			self.path = path
			self.src_db = src_db
			self.valid_nodes = set()

	porttrees_data = []
	for path in porttrees:
		src_db = portdb._pregen_auxdb.get(path)
		if src_db is None:
			# portdbapi does not populate _pregen_auxdb
			# when FEATURES=metadata-transfer is enabled
			src_db = portdb._create_pregen_cache(path)

		if src_db is not None:
			eclass_db = portdb.repositories.get_repo_for_location(path).eclass_db
			# Update eclass data which may be stale after sync.
			eclass_db.update_eclasses()
			porttrees_data.append(TreeData(portdb.auxdb[path], eclass_db, path, src_db))

	porttrees = [tree_data.path for tree_data in porttrees_data]

	quiet = settings.get('TERM') == 'dumb' or \
		'--quiet' in myopts or \
		not sys.stdout.isatty()

	onProgress = None
	if not quiet:
		progressBar = portage.output.TermProgressBar()
		progressHandler = ProgressHandler()
		onProgress = progressHandler.onProgress
		def display():
			progressBar.set(progressHandler.curval, progressHandler.maxval)
		progressHandler.display = display
		def sigwinch_handler(signum, frame):
			lines, progressBar.term_columns = \
				portage.output.get_term_size()
		signal.signal(signal.SIGWINCH, sigwinch_handler)

	# Temporarily override portdb.porttrees so portdb.cp_all()
	# will only return the relevant subset.
	portdb_porttrees = portdb.porttrees
	portdb.porttrees = porttrees
	try:
		cp_all = portdb.cp_all()
	finally:
		portdb.porttrees = portdb_porttrees

	curval = 0
	maxval = len(cp_all)
	if onProgress is not None:
		onProgress(maxval, curval)

	# TODO: Display error messages, but do not interfere with the progress bar.
	# Here's how:
	#  1) erase the progress bar
	#  2) show the error message
	#  3) redraw the progress bar on a new line

	for cp in cp_all:
		for tree_data in porttrees_data:

			src_chf = tree_data.src_db.validation_chf
			dest_chf = tree_data.dest_db.validation_chf
			dest_chf_key = '_%s_' % dest_chf
			dest_chf_getter = operator.attrgetter(dest_chf)

			for cpv in portdb.cp_list(cp, mytree=tree_data.path):
				tree_data.valid_nodes.add(cpv)
				try:
					src = tree_data.src_db[cpv]
				except (CacheError, KeyError):
					continue

				ebuild_location = portdb.findname(cpv, mytree=tree_data.path)
				if ebuild_location is None:
					continue
				ebuild_hash = hashed_path(ebuild_location)

				try:
					if not tree_data.src_db.validate_entry(src,
						ebuild_hash, tree_data.eclass_db):
						continue
				except CacheError:
					continue

				eapi = src.get('EAPI')
				if not eapi:
					eapi = '0'
				eapi_supported = eapi_is_supported(eapi)
				if not eapi_supported:
					continue

				dest = None
				try:
					dest = tree_data.dest_db[cpv]
				except (KeyError, CacheError):
					pass

				for d in (src, dest):
					if d is not None and d.get('EAPI') in ('', '0'):
						del d['EAPI']

				if src_chf != 'mtime':
					# src may contain an irrelevant _mtime_ which corresponds
					# to the time that the cache entry was written
					src.pop('_mtime_', None)

				if src_chf != dest_chf:
					# populate src entry with dest_chf_key
					# (the validity of the dest_chf that we generate from the
					# ebuild here relies on the fact that we already used
					# validate_entry to validate the ebuild with src_chf)
					src[dest_chf_key] = dest_chf_getter(ebuild_hash)

				if dest is not None:
					if not (dest.get(dest_chf_key) == src[dest_chf_key] and \
						tree_data.eclass_db.validate_and_rewrite_cache(
							dest['_eclasses_'], tree_data.dest_db.validation_chf,
							tree_data.dest_db.store_eclass_paths) is not None and \
						set(dest['_eclasses_']) == set(src['_eclasses_'])):
						dest = None
					else:
						# We don't want to skip the write unless we're really
						# sure that the existing cache is identical, so don't
						# trust _mtime_ and _eclasses_ alone.
						for k in auxdbkeys:
							if dest.get(k, '') != src.get(k, ''):
								dest = None
								break

				if dest is not None:
					# The existing data is valid and identical,
					# so there's no need to overwrite it.
					continue

				try:
					tree_data.dest_db[cpv] = src
				except CacheError:
					# ignore it; can't do anything about it.
					pass

		curval += 1
		if onProgress is not None:
			onProgress(maxval, curval)

	if onProgress is not None:
		onProgress(maxval, curval)

	for tree_data in porttrees_data:
		try:
			dead_nodes = set(tree_data.dest_db)
		except CacheError as e:
			writemsg_level("Error listing cache entries for " + \
				"'%s': %s, continuing...\n" % (tree_data.path, e),
				level=logging.ERROR, noiselevel=-1)
			del e
		else:
			dead_nodes.difference_update(tree_data.valid_nodes)
			for cpv in dead_nodes:
				try:
					del tree_data.dest_db[cpv]
				except (KeyError, CacheError):
					pass

	if not quiet:
		# make sure the final progress is displayed
		progressHandler.display()
		print()
		signal.signal(signal.SIGWINCH, signal.SIG_DFL)

	portdb.flush_cache()
	sys.stdout.flush()



# Copyright 2010-2018 Gentoo Authors
# Distributed under the terms of the GNU General Public License v2

__all__ = ['ExtractKernelVersion']

import io
import logging

from portage import os, _encodings, _unicode_encode
from portage.env.loaders import KeyValuePairFileLoader
from portage.util import grabfile, shlex_split, writemsg_level

def ExtractKernelVersion(base_dir):
	"""
	Try to figure out what kernel version we are running
	@param base_dir: Path to sources (usually /usr/src/linux)
	@type base_dir: string
	@rtype: tuple( version[string], error[string])
	@return:
	1. tuple( version[string], error[string])
	Either version or error is populated (but never both)

	"""
	lines = []
	pathname = os.path.join(base_dir, 'Makefile')
	try:
		f = io.open(_unicode_encode(pathname,
			encoding=_encodings['fs'], errors='strict'), mode='r',
			encoding=_encodings['content'], errors='replace')
	except OSError as details:
		return (None, str(details))
	except IOError as details:
		return (None, str(details))

	try:
		for i in range(4):
			lines.append(f.readline())
	except OSError as details:
		return (None, str(details))
	except IOError as details:
		return (None, str(details))
	finally:
		f.close()

	lines = [l.strip() for l in lines]

	version = ''

	#XXX: The following code relies on the ordering of vars within the Makefile
	for line in lines:
		# split on the '=' then remove annoying whitespace
		items = line.split("=")
		items = [i.strip() for i in items]
		if items[0] == 'VERSION' or \
			items[0] == 'PATCHLEVEL':
			version += items[1]
			version += "."
		elif items[0] == 'SUBLEVEL':
			version += items[1]
		elif items[0] == 'EXTRAVERSION' and \
			items[-1] != items[0]:
			version += items[1]

	# Grab a list of files named localversion* and sort them
	localversions = os.listdir(base_dir)
	for x in range(len(localversions) - 1, -1, -1):
		if localversions[x][:12] != "localversion":
			del localversions[x]
	localversions.sort()

	# Append the contents of each to the version string, stripping ALL whitespace
	for lv in localversions:
		version += "".join(" ".join(grabfile(base_dir + "/" + lv)).split())

	# Check the .config for a CONFIG_LOCALVERSION and append that too, also stripping whitespace
	loader = KeyValuePairFileLoader(os.path.join(base_dir, ".config"), None)
	kernelconfig, loader_errors = loader.load()
	if loader_errors:
		for file_path, file_errors in loader_errors.items():
			for error_str in file_errors:
				writemsg_level("%s: %s\n" % (file_path, error_str),
					level=logging.ERROR, noiselevel=-1)

	if kernelconfig and "CONFIG_LOCALVERSION" in kernelconfig:
		version += "".join(shlex_split(kernelconfig["CONFIG_LOCALVERSION"]))

	return (version, None)



# Copyright 2014 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

doc = """SVN plug-in module for portage.
Performs a svn up on repositories."""
__doc__ = doc[:]

from portage.localization import _
from portage.sync.config_checks import CheckSyncConfig
from portage.util import writemsg_level


module_spec = {
	'name': 'svn',
	'description': doc,
	'provides':{
		'svn-module': {
			'name': "svn",
			'sourcefile': "svn",
			'class': "SVNSync",
			'description': doc,
			'functions': ['sync', 'new', 'exists'],
			'func_desc': {
				'sync': 'Performs a svn up on the repository',
				'new': 'Creates the new repository at the specified location',
				'exists': 'Returns a boolean of whether the specified dir ' +
					'exists and is a valid SVN repository',
			},
			'validate_config': CheckSyncConfig,
			'module_specific_options': (),
		}
	}
}



# Copyright 1999-2009 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

from _emerge.DepPriority import DepPriority
from _emerge.Package import Package

def _find_deep_system_runtime_deps(graph):
	deep_system_deps = set()
	node_stack = []
	for node in graph:
		if not isinstance(node, Package) or \
			node.operation == 'uninstall':
			continue
		if node.root_config.sets['system'].findAtomForPackage(node):
			node_stack.append(node)

	def ignore_priority(priority):
		"""
		Ignore non-runtime priorities.
		"""
		if isinstance(priority, DepPriority) and \
			(priority.runtime or priority.runtime_post):
			return False
		return True

	while node_stack:
		node = node_stack.pop()
		if node in deep_system_deps:
			continue
		deep_system_deps.add(node)
		for child in graph.child_nodes(node, ignore_priority=ignore_priority):
			if not isinstance(child, Package) or \
				child.operation == 'uninstall':
				continue
			node_stack.append(child)

	return deep_system_deps



# Copyright 2018-2021 Gentoo Authors
# Distributed under the terms of the GNU General Public License v2

import os
import subprocess

from portage.process import find_binary
from portage.tests import TestCase
from portage.util._eventloop.global_event_loop import global_event_loop
from portage.util.futures import asyncio
from portage.util.futures._asyncio import create_subprocess_exec
from portage.util.futures.unix_events import DefaultEventLoopPolicy


class SubprocessExecTestCase(TestCase):
	def _run_test(self, test):
		initial_policy = asyncio.get_event_loop_policy()
		if not isinstance(initial_policy, DefaultEventLoopPolicy):
			asyncio.set_event_loop_policy(DefaultEventLoopPolicy())

		loop = asyncio._wrap_loop()
		try:
			test(loop)
		finally:
			asyncio.set_event_loop_policy(initial_policy)
			if loop not in (None, global_event_loop()):
				loop.close()
				self.assertFalse(global_event_loop().is_closed())

	def testEcho(self):
		args_tuple = (b'hello', b'world')
		echo_binary = find_binary("echo")
		self.assertNotEqual(echo_binary, None)
		echo_binary = echo_binary.encode()

		def test(loop):

			async def test_coroutine():

				proc = await create_subprocess_exec(echo_binary, *args_tuple,
					stdout=subprocess.PIPE, stderr=subprocess.STDOUT)

				out, err = await proc.communicate()
				self.assertEqual(tuple(out.split()), args_tuple)
				self.assertEqual(proc.returncode, os.EX_OK)

				proc = await create_subprocess_exec(
						'bash', '-c', 'echo foo; echo bar 1>&2;',
						stdout=subprocess.PIPE, stderr=subprocess.PIPE)

				out, err = await proc.communicate()
				self.assertEqual(out, b'foo\n')
				self.assertEqual(err, b'bar\n')
				self.assertEqual(proc.returncode, os.EX_OK)

				proc = await create_subprocess_exec(
						'bash', '-c', 'echo foo; echo bar 1>&2;',
						stdout=subprocess.PIPE, stderr=subprocess.STDOUT)

				out, err = await proc.communicate()
				self.assertEqual(out, b'foo\nbar\n')
				self.assertEqual(err, None)
				self.assertEqual(proc.returncode, os.EX_OK)

				return 'success'

			self.assertEqual('success',
				loop.run_until_complete(test_coroutine()))

		self._run_test(test)

	def testCat(self):
		stdin_data = b'hello world'
		cat_binary = find_binary("cat")
		self.assertNotEqual(cat_binary, None)
		cat_binary = cat_binary.encode()

		def test(loop):
			proc = loop.run_until_complete(
				create_subprocess_exec(cat_binary,
				stdin=subprocess.PIPE, stdout=subprocess.PIPE,
				loop=loop))

			out, err = loop.run_until_complete(proc.communicate(input=stdin_data))

			self.assertEqual(loop.run_until_complete(proc.wait()), os.EX_OK)
			self.assertEqual(out, stdin_data)

		self._run_test(test)

	def testPipe(self):
		stdin_data = b'hello world'
		cat_binary = find_binary("cat")
		self.assertNotEqual(cat_binary, None)
		cat_binary = cat_binary.encode()

		echo_binary = find_binary("echo")
		self.assertNotEqual(echo_binary, None)
		echo_binary = echo_binary.encode()

		def test(loop):

			pr, pw = os.pipe()

			cat_proc = loop.run_until_complete(create_subprocess_exec(
				cat_binary, stdin=pr, stdout=subprocess.PIPE,
				loop=loop))

			echo_proc = loop.run_until_complete(create_subprocess_exec(
				echo_binary, b'-n', stdin_data, stdout=pw,
				loop=loop))

			os.close(pr)
			os.close(pw)

			out, err = loop.run_until_complete(cat_proc.communicate())

			self.assertEqual(loop.run_until_complete(cat_proc.wait()), os.EX_OK)
			self.assertEqual(loop.run_until_complete(echo_proc.wait()), os.EX_OK)
			self.assertEqual(out, stdin_data)

		self._run_test(test)

	def testReadTransport(self):
		"""
		Test asyncio.create_subprocess_exec(stdout=subprocess.PIPE) which
		requires an AbstractEventLoop.connect_read_pipe implementation
		(and a ReadTransport implementation for it to return).
		"""

		args_tuple = (b'hello', b'world')
		echo_binary = find_binary("echo")
		self.assertNotEqual(echo_binary, None)
		echo_binary = echo_binary.encode()

		def test(loop):
			with open(os.devnull, 'rb', 0) as devnull:
				proc = loop.run_until_complete(
					create_subprocess_exec(
					echo_binary, *args_tuple,
					stdin=devnull,
					stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
					loop=loop))

			self.assertEqual(
				tuple(loop.run_until_complete(proc.stdout.read()).split()),
				args_tuple)
			self.assertEqual(loop.run_until_complete(proc.wait()), os.EX_OK)

		self._run_test(test)

	def testWriteTransport(self):
		"""
		Test asyncio.create_subprocess_exec(stdin=subprocess.PIPE) which
		requires an AbstractEventLoop.connect_write_pipe implementation
		(and a WriteTransport implementation for it to return).
		"""

		stdin_data = b'hello world'
		cat_binary = find_binary("cat")
		self.assertNotEqual(cat_binary, None)
		cat_binary = cat_binary.encode()

		def test(loop):
			proc = loop.run_until_complete(
				create_subprocess_exec(
				cat_binary,
				stdin=subprocess.PIPE,
				stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
				loop=loop))

			# This buffers data when necessary to avoid blocking.
			proc.stdin.write(stdin_data)
			# Any buffered data is written asynchronously after the
			# close method is called.
			proc.stdin.close()

			self.assertEqual(
				loop.run_until_complete(proc.stdout.read()),
				stdin_data)
			self.assertEqual(loop.run_until_complete(proc.wait()), os.EX_OK)

		self._run_test(test)



# Copyright 2018-2020 Gentoo Authors
# Distributed under the terms of the GNU General Public License v2

import os

from portage.tests import TestCase
from portage.util._async.AsyncFunction import AsyncFunction
from portage.util._eventloop.global_event_loop import global_event_loop
from portage.util.futures import asyncio
from portage.util.futures.unix_events import DefaultEventLoopPolicy


def fork_main():
	loop = asyncio._wrap_loop()
	# This fails with python's default event loop policy,
	# see https://bugs.python.org/issue22087.
	loop.run_until_complete(asyncio.sleep(0.1, loop=loop))
	loop.close()


def async_main(fork_exitcode, loop=None):
	loop = asyncio._wrap_loop(loop)
	proc = AsyncFunction(scheduler=loop, target=fork_main)
	proc.start()
	proc.async_wait().add_done_callback(lambda future: fork_exitcode.set_result(future.result()))


class EventLoopInForkTestCase(TestCase):
	"""
	The default asyncio event loop policy does not support loops
	running in forks, see https://bugs.python.org/issue22087.
	Portage's DefaultEventLoopPolicy supports forks.
	"""

	def testEventLoopInForkTestCase(self):
		initial_policy = asyncio.get_event_loop_policy()
		if not isinstance(initial_policy, DefaultEventLoopPolicy):
			asyncio.set_event_loop_policy(DefaultEventLoopPolicy())
		loop = None
		try:
			loop = asyncio._wrap_loop()
			fork_exitcode = loop.create_future()
			# Make async_main fork while the loop is running, which would
			# trigger https://bugs.python.org/issue22087 with asyncio's
			# default event loop policy.
			loop.call_soon(async_main, fork_exitcode)
			assert loop.run_until_complete(fork_exitcode) == os.EX_OK
		finally:
			asyncio.set_event_loop_policy(initial_policy)
			if loop not in (None, global_event_loop()):
				loop.close()
				self.assertFalse(global_event_loop().is_closed())



# Copyright 2014 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

doc = """WebRSync plug-in module for portage.
Performs a http download of a portage snapshot, verifies and
unpacks it to the repo location."""
__doc__ = doc[:]


import os

from portage.sync.config_checks import CheckSyncConfig


DEFAULT_CLASS = "WebRsync"
AVAILABLE_CLASSES = [ "WebRsync",  "PyWebsync"]
options = {"1": "WebRsync", "2": "PyWebsync"}


config_class = DEFAULT_CLASS
try:
	test_param = os.environ["TESTIT"]
	if test_param in options:
		config_class = options[test_param]
except KeyError:
	pass


module_spec = {
	'name': 'webrsync',
	'description': doc,
	'provides':{
		'webrsync-module': {
			'name': "webrsync",
			'sourcefile': "webrsync",
			'class': config_class,
			'description': doc,
			'functions': ['sync', 'new', 'exists'],
			'func_desc': {
				'sync': 'Performs an archived http download of the ' +
					'repository, then unpacks it.  Optionally it performs a ' +
					'gpg verification of the downloaded file(s)',
				'new': 'Creates the new repository at the specified location',
				'exists': 'Returns a boolean of whether the specified dir ' +
					'exists and is a valid repository',
			},
			'validate_config': CheckSyncConfig,
			'module_specific_options': (
				'sync-webrsync-delta',
				'sync-webrsync-keep-snapshots',
				'sync-webrsync-verify-signature',
			),
		},
	}
}



# Copyright 2018 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

from portage.tests import TestCase
from portage.util._eventloop.global_event_loop import global_event_loop
from portage.util.futures import asyncio
from portage.util.futures.unix_events import DefaultEventLoopPolicy


class RunUntilCompleteTestCase(TestCase):
	def test_add_done_callback(self):
		initial_policy = asyncio.get_event_loop_policy()
		if not isinstance(initial_policy, DefaultEventLoopPolicy):
			asyncio.set_event_loop_policy(DefaultEventLoopPolicy())

		loop = None
		try:
			loop = asyncio._wrap_loop()
			f1 = loop.create_future()
			f2 = loop.create_future()
			f1.add_done_callback(f2.set_result)
			loop.call_soon(lambda: f1.set_result(None))
			loop.run_until_complete(f1)
			self.assertEqual(f1.done(), True)

			# This proves that done callbacks of f1 are executed before
			# loop.run_until_complete(f1) returns, which is how asyncio's
			# default event loop behaves.
			self.assertEqual(f2.done(), True)
		finally:
			asyncio.set_event_loop_policy(initial_policy)
			if loop not in (None, global_event_loop()):
				loop.close()
				self.assertFalse(global_event_loop().is_closed())



# Copyright 2012-2019 Gentoo Authors
# Distributed under the terms of the GNU General Public License v2

from portage.tests import TestCase
from portage.tests.resolver.ResolverPlayground import (ResolverPlayground,
	ResolverPlaygroundTestCase)

class SlotAbiDowngradeTestCase(TestCase):

	def __init__(self, *args, **kwargs):
		super(SlotAbiDowngradeTestCase, self).__init__(*args, **kwargs)

	def testSubSlot(self):
		ebuilds = {
			"dev-libs/icu-4.8" : {
				"EAPI": "5",
				"SLOT": "0/48"
			},
			"dev-libs/libxml2-2.7.8" : {
				"EAPI": "5",
				"DEPEND":  "dev-libs/icu:=",
				"RDEPEND": "dev-libs/icu:="
			},
		}
		binpkgs = {
			"dev-libs/icu-49" : {
				"EAPI": "5",
				"SLOT": "0/49"
			},
			"dev-libs/icu-4.8" : {
				"EAPI": "5",
				"SLOT": "0/48"
			},
			"dev-libs/libxml2-2.7.8" : {
				"EAPI": "5",
				"DEPEND":  "dev-libs/icu:0/49=",
				"RDEPEND": "dev-libs/icu:0/49="
			},
		}
		installed = {
			"dev-libs/icu-49" : {
				"EAPI": "5",
				"SLOT": "0/49"
			},
			"dev-libs/libxml2-2.7.8" : {
				"EAPI": "5",
				"DEPEND":  "dev-libs/icu:0/49=",
				"RDEPEND": "dev-libs/icu:0/49="
			},
		}

		world = ["dev-libs/libxml2"]

		test_cases = (

			ResolverPlaygroundTestCase(
				["dev-libs/icu"],
				options = {"--oneshot": True},
				success = True,
				mergelist = ["dev-libs/icu-4.8", "dev-libs/libxml2-2.7.8" ]),

			ResolverPlaygroundTestCase(
				["dev-libs/icu"],
				options = {"--oneshot": True, "--ignore-built-slot-operator-deps": "y"},
				success = True,
				mergelist = ["dev-libs/icu-4.8"]),

			ResolverPlaygroundTestCase(
				["dev-libs/icu"],
				options = {"--oneshot": True, "--usepkg": True},
				success = True,
				mergelist = ["[binary]dev-libs/icu-4.8", "dev-libs/libxml2-2.7.8" ]),

			ResolverPlaygroundTestCase(
				["dev-libs/icu"],
				options = {"--oneshot": True, "--usepkgonly": True},
				success = True,
				mergelist = ["[binary]dev-libs/icu-49"]),

			ResolverPlaygroundTestCase(
				["@world"],
				options = {"--update": True, "--deep": True},
				success = True,
				mergelist = ["dev-libs/icu-4.8", "dev-libs/libxml2-2.7.8" ]),

			ResolverPlaygroundTestCase(
				["@world"],
				options = {"--update": True, "--deep": True, "--ignore-built-slot-operator-deps": "y"},
				success = True,
				mergelist = ["dev-libs/icu-4.8"]),

			ResolverPlaygroundTestCase(
				["@world"],
				options = {"--update": True, "--deep": True, "--usepkg": True},
				success = True,
				mergelist = ["[binary]dev-libs/icu-4.8", "dev-libs/libxml2-2.7.8" ]),

			ResolverPlaygroundTestCase(
				["@world"],
				options = {"--update": True, "--deep": True, "--usepkgonly": True},
				success = True,
				mergelist = []),

		)

		playground = ResolverPlayground(ebuilds=ebuilds, binpkgs=binpkgs,
			installed=installed, world=world, debug=False)
		try:
			for test_case in test_cases:
				playground.run_TestCase(test_case)
				self.assertEqual(test_case.test_success, True, test_case.fail_msg)
		finally:
			playground.cleanup()

	def testWholeSlotSubSlotMix(self):
		ebuilds = {
			"dev-libs/glib-1.2.10" : {
				"SLOT": "1"
			},
			"dev-libs/glib-2.30.2" : {
				"EAPI": "5",
				"SLOT": "2/2.30"
			},
			"dev-libs/dbus-glib-0.98" : {
				"EAPI": "5",
				"DEPEND":  "dev-libs/glib:2=",
				"RDEPEND": "dev-libs/glib:2="
			},
		}
		binpkgs = {
			"dev-libs/glib-1.2.10" : {
				"SLOT": "1"
			},
			"dev-libs/glib-2.30.2" : {
				"EAPI": "5",
				"SLOT": "2/2.30"
			},
			"dev-libs/glib-2.32.3" : {
				"EAPI": "5",
				"SLOT": "2/2.32"
			},
			"dev-libs/dbus-glib-0.98" : {
				"EAPI": "5",
				"DEPEND":  "dev-libs/glib:2/2.32=",
				"RDEPEND": "dev-libs/glib:2/2.32="
			},
		}
		installed = {
			"dev-libs/glib-1.2.10" : {
				"EAPI": "5",
				"SLOT": "1"
			},
			"dev-libs/glib-2.32.3" : {
				"EAPI": "5",
				"SLOT": "2/2.32"
			},
			"dev-libs/dbus-glib-0.98" : {
				"EAPI": "5",
				"DEPEND":  "dev-libs/glib:2/2.32=",
				"RDEPEND": "dev-libs/glib:2/2.32="
			},
		}

		world = ["dev-libs/glib:1", "dev-libs/dbus-glib"]

		test_cases = (

			ResolverPlaygroundTestCase(
				["dev-libs/glib"],
				options = {"--oneshot": True},
				success = True,
				mergelist = ["dev-libs/glib-2.30.2", "dev-libs/dbus-glib-0.98" ]),

			ResolverPlaygroundTestCase(
				["dev-libs/glib"],
				options = {"--oneshot": True, "--ignore-built-slot-operator-deps": "y"},
				success = True,
				mergelist = ["dev-libs/glib-2.30.2"]),

			ResolverPlaygroundTestCase(
				["dev-libs/glib"],
				options = {"--oneshot": True, "--usepkg": True},
				success = True,
				mergelist = ["[binary]dev-libs/glib-2.30.2", "dev-libs/dbus-glib-0.98" ]),

			ResolverPlaygroundTestCase(
				["dev-libs/glib"],
				options = {"--oneshot": True, "--usepkgonly": True},
				success = True,
				mergelist = ["[binary]dev-libs/glib-2.32.3"]),

			ResolverPlaygroundTestCase(
				["@world"],
				options = {"--update": True, "--deep": True},
				success = True,
				mergelist = ["dev-libs/glib-2.30.2", "dev-libs/dbus-glib-0.98" ]),

			ResolverPlaygroundTestCase(
				["@world"],
				options = {"--update": True, "--deep": True, "--ignore-built-slot-operator-deps": "y"},
				success = True,
				mergelist = ["dev-libs/glib-2.30.2"]),

			ResolverPlaygroundTestCase(
				["@world"],
				options = {"--update": True, "--deep": True, "--usepkg": True},
				success = True,
				mergelist = ["[binary]dev-libs/glib-2.30.2", "dev-libs/dbus-glib-0.98" ]),

			ResolverPlaygroundTestCase(
				["@world"],
				options = {"--update": True, "--deep": True, "--usepkgonly": True},
				success = True,
				mergelist = []),

		)

		playground = ResolverPlayground(ebuilds=ebuilds, binpkgs=binpkgs,
			installed=installed, world=world, debug=False)
		try:
			for test_case in test_cases:
				playground.run_TestCase(test_case)
				self.assertEqual(test_case.test_success, True, test_case.fail_msg)
		finally:
			playground.cleanup()



# Copyright 2011-2019 Gentoo Authors
# Distributed under the terms of the GNU General Public License v2

from portage.tests import TestCase
from portage.tests.resolver.ResolverPlayground import (ResolverPlayground,
	ResolverPlaygroundTestCase)

class KeywordsTestCase(TestCase):

	def testStableConfig(self):
		# Only accept stable keywords for a particular ARCH.

		user_config = {
			'package.accept_keywords':
				(
					'*/* -* x86',
				),
		}

		ebuilds = {
			'app-misc/A-1': {'KEYWORDS': 'x86'},
			'app-misc/B-1': {'KEYWORDS': '~x86'},
			'app-misc/C-1': {'KEYWORDS': '*'},
			'app-misc/D-1': {'KEYWORDS': '~*'},
			'app-misc/E-1': {'KEYWORDS': 'arm'},
			'app-misc/F-1': {'KEYWORDS': '~arm'},
			'app-misc/G-1': {'KEYWORDS': ''},
		}

		test_cases = (

			ResolverPlaygroundTestCase(
				['app-misc/A'],
				success = True,
				mergelist = ['app-misc/A-1']),

			ResolverPlaygroundTestCase(
				['app-misc/B'],
				success = False,
				options={'--autounmask': True},
				unstable_keywords = ('app-misc/B-1',),
				mergelist = ['app-misc/B-1']),

			ResolverPlaygroundTestCase(
				['app-misc/C'],
				success = True,
				mergelist = ['app-misc/C-1']),

			ResolverPlaygroundTestCase(
				['app-misc/D'],
				success = False,
				options={'--autounmask': True},
				unstable_keywords = ('app-misc/D-1',),
				mergelist = ['app-misc/D-1']),

			ResolverPlaygroundTestCase(
				['app-misc/E'],
				success = False,
				options={'--autounmask': True},
				unstable_keywords = ('app-misc/E-1',),
				mergelist = ['app-misc/E-1']),

			ResolverPlaygroundTestCase(
				['app-misc/F'],
				success = False,
				options={'--autounmask': True},
				unstable_keywords = ('app-misc/F-1',),
				mergelist = ['app-misc/F-1']),

			ResolverPlaygroundTestCase(
				['app-misc/G'],
				success = False,
				options={'--autounmask': True},
				unstable_keywords = ('app-misc/G-1',),
				mergelist = ['app-misc/G-1']),
		)

		playground = ResolverPlayground(ebuilds=ebuilds,
			user_config=user_config)
		try:
			for test_case in test_cases:
				playground.run_TestCase(test_case)
				self.assertEqual(test_case.test_success, True, test_case.fail_msg)
		finally:
			playground.cleanup()

	def testAnyStableConfig(self):
		# Accept stable keywords for any ARCH.

		user_config = {
			'package.accept_keywords':
				(
					'*/* -* *',
				),
		}

		ebuilds = {
			'app-misc/A-1': {'KEYWORDS': 'x86'},
			'app-misc/B-1': {'KEYWORDS': '~x86'},
			'app-misc/C-1': {'KEYWORDS': '*'},
			'app-misc/D-1': {'KEYWORDS': '~*'},
			'app-misc/E-1': {'KEYWORDS': 'arm'},
			'app-misc/F-1': {'KEYWORDS': '~arm'},
			'app-misc/G-1': {'KEYWORDS': ''},
		}

		test_cases = (

			ResolverPlaygroundTestCase(
				['app-misc/A'],
				success = True,
				mergelist = ['app-misc/A-1']),

			ResolverPlaygroundTestCase(
				['app-misc/B'],
				success = False,
				options={'--autounmask': True},
				unstable_keywords = ('app-misc/B-1',),
				mergelist = ['app-misc/B-1']),

			ResolverPlaygroundTestCase(
				['app-misc/C'],
				success = True,
				mergelist = ['app-misc/C-1']),

			ResolverPlaygroundTestCase(
				['app-misc/D'],
				success = False,
				options={'--autounmask': True},
				unstable_keywords = ('app-misc/D-1',),
				mergelist = ['app-misc/D-1']),

			ResolverPlaygroundTestCase(
				['app-misc/E'],
				success = True,
				mergelist = ['app-misc/E-1']),

			ResolverPlaygroundTestCase(
				['app-misc/F'],
				success = False,
				options={'--autounmask': True},
				unstable_keywords = ('app-misc/F-1',),
				mergelist = ['app-misc/F-1']),

			ResolverPlaygroundTestCase(
				['app-misc/G'],
				success = False,
				options={'--autounmask': True},
				unstable_keywords = ('app-misc/G-1',),
				mergelist = ['app-misc/G-1']),
		)

		playground = ResolverPlayground(ebuilds=ebuilds,
			user_config=user_config)
		try:
			for test_case in test_cases:
				playground.run_TestCase(test_case)
				self.assertEqual(test_case.test_success, True, test_case.fail_msg)
		finally:
			playground.cleanup()

	def testUnstableConfig(self):
		# Accept stable and unstable keywords for a particular ARCH.

		user_config = {
			'package.accept_keywords':
				(
					'*/* -* x86 ~x86',
				),
		}

		ebuilds = {
			'app-misc/A-1': {'KEYWORDS': 'x86'},
			'app-misc/B-1': {'KEYWORDS': '~x86'},
			'app-misc/C-1': {'KEYWORDS': '*'},
			'app-misc/D-1': {'KEYWORDS': '~*'},
			'app-misc/E-1': {'KEYWORDS': 'arm'},
			'app-misc/F-1': {'KEYWORDS': '~arm'},
			'app-misc/G-1': {'KEYWORDS': ''},
		}

		test_cases = (

			ResolverPlaygroundTestCase(
				['app-misc/A'],
				success = True,
				mergelist = ['app-misc/A-1']),

			ResolverPlaygroundTestCase(
				['app-misc/B'],
				success = True,
				mergelist = ['app-misc/B-1']),

			ResolverPlaygroundTestCase(
				['app-misc/C'],
				success = True,
				mergelist = ['app-misc/C-1']),

			ResolverPlaygroundTestCase(
				['app-misc/D'],
				success = True,
				mergelist = ['app-misc/D-1']),

			ResolverPlaygroundTestCase(
				['app-misc/E'],
				success = False,
				options={'--autounmask': True},
				unstable_keywords = ('app-misc/E-1',),
				mergelist = ['app-misc/E-1']),

			ResolverPlaygroundTestCase(
				['app-misc/F'],
				success = False,
				options={'--autounmask': True},
				unstable_keywords = ('app-misc/F-1',),
				mergelist = ['app-misc/F-1']),

			ResolverPlaygroundTestCase(
				['app-misc/G'],
				success = False,
				options={'--autounmask': True},
				unstable_keywords = ('app-misc/G-1',),
				mergelist = ['app-misc/G-1']),
		)

		playground = ResolverPlayground(ebuilds=ebuilds,
			user_config=user_config)
		try:
			for test_case in test_cases:
				playground.run_TestCase(test_case)
				self.assertEqual(test_case.test_success, True, test_case.fail_msg)
		finally:
			playground.cleanup()

	def testAnyUnstableConfig(self):
		# Accept unstable keywords for any ARCH.

		user_config = {
			'package.accept_keywords':
				(
					'*/* -* * ~*',
				),
		}

		ebuilds = {
			'app-misc/A-1': {'KEYWORDS': 'x86'},
			'app-misc/B-1': {'KEYWORDS': '~x86'},
			'app-misc/C-1': {'KEYWORDS': '*'},
			'app-misc/D-1': {'KEYWORDS': '~*'},
			'app-misc/E-1': {'KEYWORDS': 'arm'},
			'app-misc/F-1': {'KEYWORDS': '~arm'},
			'app-misc/G-1': {'KEYWORDS': ''},
		}

		test_cases = (

			ResolverPlaygroundTestCase(
				['app-misc/A'],
				success = True,
				mergelist = ['app-misc/A-1']),

			ResolverPlaygroundTestCase(
				['app-misc/B'],
				success = True,
				mergelist = ['app-misc/B-1']),

			ResolverPlaygroundTestCase(
				['app-misc/C'],
				success = True,
				mergelist = ['app-misc/C-1']),

			ResolverPlaygroundTestCase(
				['app-misc/D'],
				success = True,
				mergelist = ['app-misc/D-1']),

			ResolverPlaygroundTestCase(
				['app-misc/E'],
				success = True,
				mergelist = ['app-misc/E-1']),

			ResolverPlaygroundTestCase(
				['app-misc/F'],
				success = True,
				mergelist = ['app-misc/F-1']),

			ResolverPlaygroundTestCase(
				['app-misc/G'],
				success = False,
				options={'--autounmask': True},
				unstable_keywords = ('app-misc/G-1',),
				mergelist = ['app-misc/G-1']),
		)

		playground = ResolverPlayground(ebuilds=ebuilds,
			user_config=user_config)
		try:
			for test_case in test_cases:
				playground.run_TestCase(test_case)
				self.assertEqual(test_case.test_success, True, test_case.fail_msg)
		finally:
			playground.cleanup()

	def testIgnoreKeywordsConfig(self):
		# Ignore keywords entirely (accept **)

		user_config = {
			'package.accept_keywords':
				(
					'*/* -* **',
				),
		}

		ebuilds = {
			'app-misc/A-1': {'KEYWORDS': 'x86'},
			'app-misc/B-1': {'KEYWORDS': '~x86'},
			'app-misc/C-1': {'KEYWORDS': '*'},
			'app-misc/D-1': {'KEYWORDS': '~*'},
			'app-misc/E-1': {'KEYWORDS': 'arm'},
			'app-misc/F-1': {'KEYWORDS': '~arm'},
			'app-misc/G-1': {'KEYWORDS': ''},
		}

		test_cases = (

			ResolverPlaygroundTestCase(
				['app-misc/A'],
				success = True,
				mergelist = ['app-misc/A-1']),

			ResolverPlaygroundTestCase(
				['app-misc/B'],
				success = True,
				mergelist = ['app-misc/B-1']),

			ResolverPlaygroundTestCase(
				['app-misc/C'],
				success = True,
				mergelist = ['app-misc/C-1']),

			ResolverPlaygroundTestCase(
				['app-misc/D'],
				success = True,
				mergelist = ['app-misc/D-1']),

			ResolverPlaygroundTestCase(
				['app-misc/E'],
				success = True,
				mergelist = ['app-misc/E-1']),

			ResolverPlaygroundTestCase(
				['app-misc/F'],
				success = True,
				mergelist = ['app-misc/F-1']),

			ResolverPlaygroundTestCase(
				['app-misc/G'],
				success = True,
				mergelist = ['app-misc/G-1']),
		)

		playground = ResolverPlayground(ebuilds=ebuilds,
			user_config=user_config)
		try:
			for test_case in test_cases:
				playground.run_TestCase(test_case)
				self.assertEqual(test_case.test_success, True, test_case.fail_msg)
		finally:
			playground.cleanup()



# Copyright 1999-2013 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

import gc

def clear_caches(trees):
	for d in trees.values():
		d["porttree"].dbapi.melt()
		d["porttree"].dbapi._aux_cache.clear()
		d["bintree"].dbapi._clear_cache()
		if d["vartree"].dbapi._linkmap is None:
			# preserve-libs is entirely disabled
			pass
		else:
			d["vartree"].dbapi._linkmap._clear_cache()
	gc.collect()



# Copyright 2018-2021 Gentoo Authors
# Distributed under the terms of the GNU General Public License v2

from concurrent.futures import Future, ThreadPoolExecutor
import contextlib

try:
	import threading
except ImportError:
	import dummy_threading as threading

import weakref
import time

import portage
from portage.tests import TestCase
from portage.util._eventloop.global_event_loop import global_event_loop
from portage.util.backoff import RandomExponentialBackoff
from portage.util.futures import asyncio
from portage.util.futures.retry import retry
from portage.util.futures.executor.fork import ForkExecutor


class SucceedLaterException(Exception):
	pass


class SucceedLater:
	"""
	A callable object that succeeds some duration of time has passed.
	"""
	def __init__(self, duration):
		self._succeed_time = time.monotonic() + duration

	async def __call__(self):
		remaining = self._succeed_time - time.monotonic()
		if remaining > 0:
			await asyncio.sleep(remaining)
		return 'success'


class SucceedNeverException(Exception):
	pass


class SucceedNever:
	"""
	A callable object that never succeeds.
	"""
	async def __call__(self):
		raise SucceedNeverException('expected failure')


class HangForever:
	"""
	A callable object that sleeps forever.
	"""
	async def __call__(self):
		while True:
			await asyncio.sleep(9)


class RetryTestCase(TestCase):

	@contextlib.contextmanager
	def _wrap_coroutine_func(self, coroutine_func):
		"""
		Derived classes may override this method in order to implement
		alternative forms of execution.
		"""
		yield coroutine_func

	def testSucceedLater(self):
		loop = global_event_loop()
		with self._wrap_coroutine_func(SucceedLater(1)) as func_coroutine:
			decorator = retry(try_max=9999,
				delay_func=RandomExponentialBackoff(multiplier=0.1, base=2))
			decorated_func = decorator(func_coroutine, loop=loop)
			result = loop.run_until_complete(decorated_func())
			self.assertEqual(result, 'success')

	def testSucceedNever(self):
		loop = global_event_loop()
		with self._wrap_coroutine_func(SucceedNever()) as func_coroutine:
			decorator = retry(try_max=4, try_timeout=None,
				delay_func=RandomExponentialBackoff(multiplier=0.1, base=2))
			decorated_func = decorator(func_coroutine, loop=loop)
			done, pending = loop.run_until_complete(asyncio.wait([decorated_func()], loop=loop))
			self.assertEqual(len(done), 1)
			self.assertTrue(isinstance(done.pop().exception().__cause__, SucceedNeverException))

	def testSucceedNeverReraise(self):
		loop = global_event_loop()
		with self._wrap_coroutine_func(SucceedNever()) as func_coroutine:
			decorator = retry(reraise=True, try_max=4, try_timeout=None,
				delay_func=RandomExponentialBackoff(multiplier=0.1, base=2))
			decorated_func = decorator(func_coroutine, loop=loop)
			done, pending = loop.run_until_complete(asyncio.wait([decorated_func()], loop=loop))
			self.assertEqual(len(done), 1)
			self.assertTrue(isinstance(done.pop().exception(), SucceedNeverException))

	def testHangForever(self):
		loop = global_event_loop()
		with self._wrap_coroutine_func(HangForever()) as func_coroutine:
			decorator = retry(try_max=2, try_timeout=0.1,
				delay_func=RandomExponentialBackoff(multiplier=0.1, base=2))
			decorated_func = decorator(func_coroutine, loop=loop)
			done, pending = loop.run_until_complete(asyncio.wait([decorated_func()], loop=loop))
			self.assertEqual(len(done), 1)
			self.assertTrue(isinstance(done.pop().exception().__cause__, asyncio.TimeoutError))

	def testHangForeverReraise(self):
		loop = global_event_loop()
		with self._wrap_coroutine_func(HangForever()) as func_coroutine:
			decorator = retry(reraise=True, try_max=2, try_timeout=0.1,
				delay_func=RandomExponentialBackoff(multiplier=0.1, base=2))
			decorated_func = decorator(func_coroutine, loop=loop)
			done, pending = loop.run_until_complete(asyncio.wait([decorated_func()], loop=loop))
			self.assertEqual(len(done), 1)
			self.assertTrue(isinstance(done.pop().exception(), asyncio.TimeoutError))

	def testCancelRetry(self):
		loop = global_event_loop()
		with self._wrap_coroutine_func(SucceedNever()) as func_coroutine:
			decorator = retry(try_timeout=0.1,
				delay_func=RandomExponentialBackoff(multiplier=0.1, base=2))
			decorated_func = decorator(func_coroutine, loop=loop)
			future = decorated_func()
			loop.call_later(0.3, future.cancel)
			done, pending = loop.run_until_complete(asyncio.wait([future], loop=loop))
			self.assertEqual(len(done), 1)
			self.assertTrue(done.pop().cancelled())

	def testOverallTimeoutWithException(self):
		loop = global_event_loop()
		with self._wrap_coroutine_func(SucceedNever()) as func_coroutine:
			decorator = retry(try_timeout=0.1, overall_timeout=0.3,
				delay_func=RandomExponentialBackoff(multiplier=0.1, base=2))
			decorated_func = decorator(func_coroutine, loop=loop)
			done, pending = loop.run_until_complete(asyncio.wait([decorated_func()], loop=loop))
			self.assertEqual(len(done), 1)
			self.assertTrue(isinstance(done.pop().exception().__cause__, SucceedNeverException))

	def testOverallTimeoutWithTimeoutError(self):
		loop = global_event_loop()
		# results in TimeoutError because it hangs forever
		with self._wrap_coroutine_func(HangForever()) as func_coroutine:
			decorator = retry(try_timeout=0.1, overall_timeout=0.3,
				delay_func=RandomExponentialBackoff(multiplier=0.1, base=2))
			decorated_func = decorator(func_coroutine, loop=loop)
			done, pending = loop.run_until_complete(asyncio.wait([decorated_func()], loop=loop))
			self.assertEqual(len(done), 1)
			self.assertTrue(isinstance(done.pop().exception().__cause__, asyncio.TimeoutError))


class RetryForkExecutorTestCase(RetryTestCase):
	"""
	Wrap each coroutine function with AbstractEventLoop.run_in_executor,
	in order to test the event loop's default executor. The executor
	may use either a thread or a subprocess, and either case is
	automatically detected and handled.
	"""
	def __init__(self, *pargs, **kwargs):
		super(RetryForkExecutorTestCase, self).__init__(*pargs, **kwargs)
		self._executor = None

	def _setUpExecutor(self):
		self._executor = ForkExecutor()

	def _tearDownExecutor(self):
		if self._executor is not None:
			self._executor.shutdown(wait=True)
			self._executor = None

	def setUp(self):
		self._setUpExecutor()

	def tearDown(self):
		self._tearDownExecutor()

	@contextlib.contextmanager
	def _wrap_coroutine_func(self, coroutine_func):
		parent_loop = global_event_loop()
		parent_pid = portage.getpid()
		pending = weakref.WeakValueDictionary()

		# Since ThreadPoolExecutor does not propagate cancellation of a
		# parent_future to the underlying coroutine, use kill_switch to
		# propagate task cancellation to wrapper, so that HangForever's
		# thread returns when retry eventually cancels parent_future.
		def wrapper(kill_switch):
			if portage.getpid() == parent_pid:
				# thread in main process
				def done_callback(result):
					result.cancelled() or result.exception() or result.result()
					kill_switch.set()
				def start_coroutine(future):
					result = asyncio.ensure_future(coroutine_func(), loop=parent_loop)
					pending[id(result)] = result
					result.add_done_callback(done_callback)
					future.set_result(result)
				future = Future()
				parent_loop.call_soon_threadsafe(start_coroutine, future)
				kill_switch.wait()
				if not future.done():
					future.cancel()
					raise asyncio.CancelledError
				elif not future.result().done():
					future.result().cancel()
					raise asyncio.CancelledError
				else:
					return future.result().result()

			# child process
			loop = global_event_loop()
			try:
				return loop.run_until_complete(coroutine_func())
			finally:
				loop.close()

		def execute_wrapper():
			kill_switch = threading.Event()
			parent_future = asyncio.ensure_future(
				parent_loop.run_in_executor(self._executor, wrapper, kill_switch),
				loop=parent_loop)
			def kill_callback(parent_future):
				if not kill_switch.is_set():
					kill_switch.set()
			parent_future.add_done_callback(kill_callback)
			return parent_future

		try:
			yield execute_wrapper
		finally:
			while True:
				try:
					_, future = pending.popitem()
				except KeyError:
					break
				try:
					parent_loop.run_until_complete(future)
				except (Exception, asyncio.CancelledError):
					pass
				future.cancelled() or future.exception() or future.result()


class RetryThreadExecutorTestCase(RetryForkExecutorTestCase):
	def _setUpExecutor(self):
		self._executor = ThreadPoolExecutor(max_workers=1)



# test_doins.py -- Portage Unit Testing Functionality
# Copyright 2007-2010 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

import grp
import os
import pwd
import stat

from portage.tests.bin import setup_env
from portage import tests

doins = setup_env.doins
exists_in_D = setup_env.exists_in_D


class DoIns(setup_env.BinTestCase):
	def testDoIns(self):
		"""Tests the most basic senario."""
		self.init()
		try:
			env = setup_env.env
			# Create a file to be installed.
			test_path = os.path.join(env['S'], 'test')
			with open(test_path, 'w'):
				pass
			doins('test')
			exists_in_D('/test')
			st = os.lstat(env['D'] + '/test')
			# By default, `install`'s permission is 755.
			if stat.S_IMODE(st.st_mode) != 0o755:
				raise tests.TestCase.failureException
		finally:
			self.cleanup()

	def testDoInsOption(self):
		"""Tests with INSOPTIONS doins.py understands."""
		self.init()
		try:
			env = setup_env.env
			env['INSOPTIONS'] = '-pm0644'
			with open(os.path.join(env['S'], 'test'), 'w'):
				pass
			doins('test')
			st = os.lstat(env['D'] + '/test')
			if stat.S_IMODE(st.st_mode) != 0o644:
				raise tests.TestCase.failureException
			self.assertEqual(
				os.stat(os.path.join(env['S'], 'test'))[stat.ST_MTIME],
				st[stat.ST_MTIME])
		finally:
			self.cleanup()

	def testDoInsOptionUnsupportedMode(self):
		"""Tests with INSOPTIONS in the format doins.py doesn't know."""
		self.init()
		try:
			env = setup_env.env
			# Parse test for -m with unsupported format.
			# It should fall back to `install` command.
			env['INSOPTIONS'] = '-m u+r'
			with open(os.path.join(env['S'], 'test'), 'w'):
				pass
			doins('test')
			st = os.lstat(env['D'] + '/test')
			if stat.S_IMODE(st.st_mode) != 0o400:
				raise tests.TestCase.failureException
		finally:
			self.cleanup()

	def testDoInsOptionUid(self):
		"""Tests setting owner by uid works."""
		self.init()
		try:
			env = setup_env.env
			with open(os.path.join(env['S'], 'test'), 'w'):
				pass
			uid = os.lstat(os.path.join(env['S'], 'test')).st_uid
			# Set owner option with uid. No guarantee that this
			# runs with capability, we set the current UID so that
			# chown should success, although it is difficult to
			# check if chown actually runs or not.
			env['INSOPTIONS'] = '-o %d' % uid
			doins('test')
			st = os.lstat(env['D'] + '/test')
			if st.st_uid != uid:
				raise tests.TestCase.failureException
		finally:
			self.cleanup()

	def testDoInsOptionUserName(self):
		"""Tests setting owner by name works."""
		self.init()
		try:
			env = setup_env.env
			with open(os.path.join(env['S'], 'test'), 'w'):
				pass
			uid = os.lstat(os.path.join(env['S'], 'test')).st_uid
			pw = pwd.getpwuid(uid)
			# Similary to testDoInsOptionUid, use user name.
			env['INSOPTIONS'] = '-o %s' % pw.pw_name
			doins('test')
			st = os.lstat(env['D'] + '/test')
			if st.st_uid != uid:
				raise tests.TestCase.failureException
		finally:
			self.cleanup()

	def testDoInsOptionGid(self):
		"""Tests setting group by gid works."""
		self.init()
		try:
			env = setup_env.env
			with open(os.path.join(env['S'], 'test'), 'w'):
				pass
			gid = os.lstat(os.path.join(env['S'], 'test')).st_gid
			# Similary to testDoInsOptionUid, use gid.
			env['INSOPTIONS'] = '-g %d' % gid
			doins('test')
			st = os.lstat(env['D'] + '/test')
			if st.st_gid != gid:
				raise tests.TestCase.failureException
		finally:
			self.cleanup()

	def testDoInsOptionGroupName(self):
		"""Tests setting group by name works."""
		self.init()
		try:
			env = setup_env.env
			with open(os.path.join(env['S'], 'test'), 'w'):
				pass
			gid = os.lstat(os.path.join(env['S'], 'test')).st_gid
			gr = grp.getgrgid(gid)
			# Similary to testDoInsOptionUid, use group name.
			env['INSOPTIONS'] = '-g %s' % gr.gr_name
			doins('test')
			st = os.lstat(env['D'] + '/test')
			if st.st_gid != gid:
				raise tests.TestCase.failureException
		finally:
			self.cleanup()

	def testDoInsFallback(self):
		"""Tests if falling back to the `install` command works."""
		self.init()
		try:
			env = setup_env.env
			# Use an option which doins.py does not know.
			# Then, fallback to `install` command is expected.
			env['INSOPTIONS'] = '-b'
			with open(os.path.join(env['S'], 'test'), 'w'):
				pass
			doins('test')
			# So `install` should still work.
			exists_in_D('/test')
		finally:
			self.cleanup()

	def testDoInsRecursive(self):
		"""Tests installing a directory recursively."""
		self.init()
		try:
			env = setup_env.env
			os.mkdir(os.path.join(env['S'], 'testdir'))
			with open(os.path.join(env['S'], 'testdir/test'), 'w'):
				pass
			doins('-r testdir')
			exists_in_D('/testdir/test')
		finally:
			self.cleanup()

	def testDoDirOption(self):
		"""Tests with DIROPTIONS."""
		self.init()
		try:
			env = setup_env.env
			# Use an option which doins.py knows.
			env['DIROPTIONS'] = '-m0755'
			os.mkdir(os.path.join(env['S'], 'testdir'))
			with open(os.path.join(env['S'], 'testdir/test'), 'w'):
				pass
			doins('-r testdir')
			st = os.lstat(env['D'] + '/testdir')
			if stat.S_IMODE(st.st_mode) != 0o755:
				raise tests.TestCase.failureException
		finally:
			self.cleanup()

	def testDoDirFallback(self):
		"""Tests with DIROPTIONS which doins.py doesn't understand."""
		self.init()
		try:
			env = setup_env.env
			# Use an option which doins.py does not know.
			# Then, fallback to `install` command is expected.
			env['DIROPTIONS'] = '-p'
			os.mkdir(os.path.join(env['S'], 'testdir'))
			with open(os.path.join(env['S'], 'testdir/test'), 'w'):
				pass
			doins('-r testdir')
			# So, `install` should still work.
			exists_in_D('/testdir/test')
		finally:
			self.cleanup()

	def testSymlinkFile(self):
		"""Tests if installing a symlink works.

		In EAPI=4 and later, installing a symlink should creates a
		symlink at destination.
		"""
		self.init()
		try:
			env = setup_env.env
			env['EAPI'] = '4'  # Enable symlink.
			env['ED'] = env['D']
			env['PORTAGE_ACTUAL_DISTDIR'] = '/foo'
			# Create a file to be installed.
			test_path = os.path.join(env['S'], 'test')
			with open(test_path, 'w'):
				pass
			symlink_path = os.path.join(env['S'], 'symlink')
			os.symlink('test', symlink_path)
			doins('test symlink')
			exists_in_D('/symlink')
			# Make sure installed symlink is actually a symbolic
			# link pointing to test.
			if not os.path.islink(env['D'] + '/symlink'):
				raise tests.TestCase.failureException
			if os.readlink(env['D'] + '/symlink') != 'test':
				raise tests.TestCase.failureException
		finally:
			self.cleanup()

	def testSymlinkFileRecursive(self):
		"""Tests if installing a symlink in a directory works."""
		self.init()
		try:
			env = setup_env.env
			env['EAPI'] = '4'  # Enable symlink.
			env['ED'] = env['D']
			env['PORTAGE_ACTUAL_DISTDIR'] = '/foo'
			# Create a file to be installed.
			parent_path = os.path.join(env['S'], 'test')
			os.mkdir(parent_path)
			with open(os.path.join(parent_path, 'test'), 'w'):
				pass
			symlink_path = os.path.join(
				env['S'], 'test', 'symlink')
			os.symlink('test', symlink_path)
			doins('-r test')
			exists_in_D('/test/symlink')
			# Make sure installed symlink is actually a symbolic
			# link pointing to test.
			if not os.path.islink(env['D'] + '/test/symlink'):
				raise tests.TestCase.failureException
			if os.readlink(env['D'] + '/test/symlink') != 'test':
				raise tests.TestCase.failureException
		finally:
			self.cleanup()

	def testSymlinkDir(self):
		"""Tests installing a symlink to a directory."""
		self.init()
		try:
			env = setup_env.env
			env['EAPI'] = '4'  # Enable symlink.
			env['ED'] = env['D']
			env['PORTAGE_ACTUAL_DISTDIR'] = '/foo'
			# Create a dir to be installed.
			os.mkdir(os.path.join(env['S'], 'test'))
			symlink_path = os.path.join(env['S'], 'symlink')
			os.symlink('test', symlink_path)
			doins('test symlink')
			# Make sure installed symlink is actually a symbolic
			# link pointing to test.
			if not os.path.islink(env['D'] + '/symlink'):
				raise tests.TestCase.failureException
			if os.readlink(env['D'] + '/symlink') != 'test':
				raise tests.TestCase.failureException
		finally:
			self.cleanup()

	def testSymlinkDirRecursive(self):
		"""Tests installing a symlink to a directory under a directory.
		"""
		self.init()
		try:
			env = setup_env.env
			env['EAPI'] = '4'  # Enable symlink.
			env['ED'] = env['D']
			env['PORTAGE_ACTUAL_DISTDIR'] = '/foo'
			# Create a file to be installed.
			parent_path = os.path.join(env['S'], 'test')
			os.mkdir(parent_path)
			os.mkdir(os.path.join(parent_path, 'test'))
			symlink_path = os.path.join(
				env['S'], 'test', 'symlink')
			os.symlink('test', symlink_path)
			doins('-r test')
			# Make sure installed symlink is actually a symbolic
			# link pointing to test.
			if not os.path.islink(env['D'] + '/test/symlink'):
				raise tests.TestCase.failureException
			if not os.path.isdir(env['D'] + '/test/symlink'):
				raise tests.TestCase.failureException
			if os.readlink(env['D'] + '/test/symlink') != 'test':
				raise tests.TestCase.failureException
		finally:
			self.cleanup()

	def testSymlinkOverwrite(self):
		"""Tests installing a file to overwrite the existing file.

		Specifically, if the existing file is a symlink, it should be
		removed once, and the file content should be copied.
		"""
		self.init()
		try:
			env = setup_env.env
			test_path = os.path.join(env['S'], 'test')
			with open(test_path, 'w'):
				pass
			# Create a dangling symlink. If removal does not work,
			# this would easily cause ENOENT error.
			os.symlink('foo/bar', env['D'] + '/test')
			doins('test')
			# Actual file should be installed.
			if os.path.islink(env['D'] + '/test'):
				raise tests.TestCase.failureException
		finally:
			self.cleanup()

	def testHardlinkOverwrite(self):
		"""Tests installing a file to overwrite the hardlink.

		If the existing file is a hardlink, it should be removed once,
		and the file content should be copied.
		"""
		self.init()
		try:
			env = setup_env.env
			test_path = os.path.join(env['S'], 'test')
			with open(test_path, 'w'):
				pass
			# Create hardlink at the dest.
			os.link(test_path, env['D'] + '/test')
			doins('test')
			# The hardlink should be unlinked, and then a copy
			# should be created.
			if os.path.samefile(test_path, env['D'] + '/test'):
				raise tests.TestCase.failureException
		finally:
			self.cleanup()



# Copyright 2005-2013 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

doc = """Check and clean old logs in the PORTAGE_LOGDIR."""
__doc__ = doc


module_spec = {
	'name': 'logs',
	'description': doc,
	'provides':{
		'module1': {
			'name': "logs",
			'sourcefile': "logs",
			'class': "CleanLogs",
			'description': doc,
			'functions': ['check','clean'],
			'func_desc': {
				'clean': {
					"short": "-C", "long": "--clean",
					"help": "Cleans out logs more than 7 days old (cleanlogs only)" + \
								 "   module-options: -t, -p",
					'status': "Cleaning %s",
					'action': 'store_true',
					'func': 'clean',
					},
				'time': {
					"short": "-t", "long": "--time",
					"help": "(cleanlogs only): -t, --time   Delete logs older than NUM of days",
					'status': "",
					'type': int,
					'dest': 'NUM',
					'func': 'clean'
					},
				'pretend': {
					"short": "-p", "long": "--pretend",
					"help": "(cleanlogs only): -p, --pretend   Output logs that would be deleted",
					'status': "",
					'action': 'store_true',
					'dest': 'pretend',
					'func': 'clean'
					}
				}
			}
		}
	}



# Copyright 2010-2020 Gentoo Authors
# Distributed under the terms of the GNU General Public License v2

from portage.tests import TestCase
from portage.tests.resolver.ResolverPlayground import ResolverPlayground, ResolverPlaygroundTestCase

class SimpleResolverTestCase(TestCase):

	def testSimple(self):
		ebuilds = {
			"dev-libs/A-1": { "KEYWORDS": "x86" },
			"dev-libs/A-2": { "KEYWORDS": "~x86" },
			"dev-libs/B-1.2": {},

			"app-misc/Z-1": { "DEPEND": "|| ( app-misc/Y ( app-misc/X app-misc/W ) )", "RDEPEND": "" },
			"app-misc/Y-1": { "KEYWORDS": "~x86" },
			"app-misc/X-1": {},
			"app-misc/W-1": {},
			}
		binpkgs = {
			"dev-libs/B-1.2": {},
		}
		installed = {
			"dev-libs/A-1": {},
			"dev-libs/B-1.1": {},
			}

		test_cases = (
			ResolverPlaygroundTestCase(["dev-libs/A"], success = True, mergelist = ["dev-libs/A-1"]),
			ResolverPlaygroundTestCase(["=dev-libs/A-2"], options = { "--autounmask": 'n' }, success = False),

			ResolverPlaygroundTestCase(
				["dev-libs/A"],
				options = {"--noreplace": True},
				success = True,
				mergelist = []),
			ResolverPlaygroundTestCase(
				["dev-libs/B"],
				options = {"--noreplace": True},
				success = True,
				mergelist = []),
			ResolverPlaygroundTestCase(
				["dev-libs/B"],
				options = {"--update": True},
				success = True,
				mergelist = ["dev-libs/B-1.2"]),

			ResolverPlaygroundTestCase(
				["dev-libs/B"],
				options = {"--update": True, "--usepkg": True},
				success = True,
				mergelist = ["[binary]dev-libs/B-1.2"]),

			ResolverPlaygroundTestCase(
				["dev-libs/B"],
				options = {"--update": True, "--usepkgonly": True},
				success = True,
				mergelist = ["[binary]dev-libs/B-1.2"]),

			ResolverPlaygroundTestCase(
				["app-misc/Z"],
				success = True,
				ambiguous_merge_order = True,
				mergelist = [("app-misc/W-1", "app-misc/X-1"), "app-misc/Z-1"]),
			)

		playground = ResolverPlayground(ebuilds=ebuilds,
			binpkgs=binpkgs, installed=installed)
		try:
			for test_case in test_cases:
				playground.run_TestCase(test_case)
				self.assertEqual(test_case.test_success, True, test_case.fail_msg)
		finally:
			playground.cleanup()



#!/usr/bin/env python
# Copyright 1998-2021 Gentoo Authors
# Distributed under the terms of the GNU General Public License v2

try:
	from setuptools.core import setup, Command, Extension
	from setuptools.command.build import build
	from setuptools.command.build_ext import build_ext as _build_ext
	from setuptools.command.build_scripts import build_scripts
	from setuptools.command.clean import clean
	from setuptools.command.install import install
	from setuptools.command.install_data import install_data
	from setuptools.command.install_lib import install_lib
	from setuptools.command.install_scripts import install_scripts
	from setuptools.command.sdist import sdist
	from setuptools.dep_util import newer
	from setuptools.dir_util import mkpath, remove_tree
	from setuptools.util import change_root, subst_vars
except ImportError:
	from distutils.core import setup, Command, Extension
	from distutils.command.build import build
	from distutils.command.build_ext import build_ext as _build_ext
	from distutils.command.build_scripts import build_scripts
	from distutils.command.clean import clean
	from distutils.command.install import install
	from distutils.command.install_data import install_data
	from distutils.command.install_lib import install_lib
	from distutils.command.install_scripts import install_scripts
	from distutils.command.sdist import sdist
	from distutils.dep_util import newer
	from distutils.dir_util import mkpath, remove_tree
	from distutils.util import change_root, subst_vars

import codecs
import collections
import glob
import itertools
import os
import os.path
import platform
import re
import subprocess
import sys

autodetect_pip = os.path.basename(os.environ.get("_", "")) == "pip" or os.path.basename(
	os.path.dirname(__file__)
).startswith("pip-")
venv_prefix = "" if sys.prefix == sys.base_prefix else sys.prefix
create_entry_points = bool(autodetect_pip or venv_prefix)
with open(os.path.join(os.path.dirname(__file__), 'README'), 'rt') as f:
	long_description = f.read()

# TODO:
# - smarter rebuilds of docs w/ 'install_docbook' and 'install_apidoc'.

# Dictionary of scripts.  The structure is
#   key   = location in filesystem to install the scripts
#   value = list of scripts, path relative to top source directory
x_scripts = {
	'bin': [
		'bin/ebuild', 'bin/egencache', 'bin/emerge', 'bin/emerge-webrsync',
		'bin/emirrordist', 'bin/glsa-check', 'bin/portageq', 'bin/quickpkg',
	],
	'sbin': [
		'bin/archive-conf', 'bin/dispatch-conf', 'bin/emaint', 'bin/env-update',
		'bin/etc-update', 'bin/fixpackages', 'bin/regenworld'
	],
}

# Dictionary custom modules written in C/C++ here.  The structure is
#   key   = module name
#   value = list of C/C++ source code, path relative to top source directory
x_c_helpers = {
	'portage.util.libc' : [
		'src/portage_util_libc.c',
	],
}

if platform.system() == 'Linux':
	x_c_helpers.update({
		'portage.util.file_copy.reflink_linux': [
			'src/portage_util_file_copy_reflink_linux.c',
		],
	})


class x_build(build):
	""" Build command with extra build_man call. """

	def run(self):
		build.run(self)
		self.run_command('build_man')


class build_man(Command):
	""" Perform substitutions in manpages. """

	user_options = [
	]

	def initialize_options(self):
		self.build_base = None

	def finalize_options(self):
		self.set_undefined_options('build',
			('build_base', 'build_base'))

	def run(self):
		for d, files in self.distribution.data_files:
			if not d.startswith('$mandir/'):
				continue

			for source in files:
				target = os.path.join(self.build_base, source)
				mkpath(os.path.dirname(target))

				if not newer(source, target) and not newer(__file__, target):
					continue

				print('copying and updating %s -> %s' % (
					source, target))

				with codecs.open(source, 'r', 'utf8') as f:
					data = f.readlines()
				data[0] = data[0].replace('VERSION',
						self.distribution.get_version())
				with codecs.open(target, 'w', 'utf8') as f:
					f.writelines(data)


class docbook(Command):
	""" Build docs using docbook. """

	user_options = [
		('doc-formats=', None, 'Documentation formats to build (all xmlto formats for docbook are allowed, comma-separated'),
	]

	def initialize_options(self):
		self.doc_formats = 'xhtml,xhtml-nochunks'

	def finalize_options(self):
		self.doc_formats = self.doc_formats.replace(',', ' ').split()

	def run(self):
		if not os.path.isdir('doc/fragment'):
			mkpath('doc/fragment')

		with open('doc/fragment/date', 'w'):
			pass
		with open('doc/fragment/version', 'w') as f:
			f.write('<releaseinfo>%s</releaseinfo>' % self.distribution.get_version())

		for f in self.doc_formats:
			print('Building docs in %s format...' % f)
			subprocess.check_call(['xmlto', '-o', 'doc',
				'-m', 'doc/custom.xsl', f, 'doc/portage.docbook'])


class apidoc(Command):
	""" Build API docs using apidoc. """

	user_options = [
	]

	def initialize_options(self):
		self.build_lib = None

	def finalize_options(self):
		self.set_undefined_options('build_py', ('build_lib', 'build_lib'))

	def run(self):
		self.run_command('build_py')

		print('Building API documentation...')

		process_env = os.environ.copy()
		pythonpath = self.build_lib
		try:
			pythonpath += ':' + process_env['PYTHONPATH']
		except KeyError:
			pass
		process_env['PYTHONPATH'] = pythonpath

		subprocess.check_call(['make', '-C', 'doc/api', 'html'],
			env = process_env)


class install_docbook(install_data):
	""" install_data for docbook docs """

	user_options = install_data.user_options + [
		('htmldir=', None, "HTML documentation install directory"),
	]

	def initialize_options(self):
		install_data.initialize_options(self)
		self.htmldir = None

	def finalize_options(self):
		self.set_undefined_options('install', ('htmldir', 'htmldir'))
		install_data.finalize_options(self)

	def run(self):
		if not os.path.exists('doc/portage.html'):
			self.run_command('docbook')
		self.data_files = [
			(self.htmldir, glob.glob('doc/*.html')),
		]
		install_data.run(self)


class install_apidoc(install_data):
	""" install_data for apidoc docs """

	user_options = install_data.user_options + [
		('htmldir=', None, "HTML documentation install directory"),
	]

	def initialize_options(self):
		install_data.initialize_options(self)
		self.htmldir = None

	def finalize_options(self):
		self.set_undefined_options('install', ('htmldir', 'htmldir'))
		install_data.finalize_options(self)

	def run(self):
		if not os.path.exists('doc/api/build/html/index.html'):
			self.run_command('apidoc')
		self.data_files = [
			(os.path.join(self.htmldir, 'api'), glob.glob('doc/api/build/html/*.html') + glob.glob('doc/api/build/html/*.js')),
			(os.path.join(self.htmldir, 'api/_static'), glob.glob('doc/api/build/html/_static/*')),
		]
		install_data.run(self)


class x_build_scripts_custom(build_scripts):
	def finalize_options(self):
		build_scripts.finalize_options(self)
		if 'dir_name' in dir(self):
			self.build_dir = os.path.join(self.build_dir, self.dir_name)
			if self.dir_name in x_scripts:
				self.scripts = x_scripts[self.dir_name]
			else:
				self.scripts = set(self.scripts)
				if not (create_entry_points and self.dir_name == "portage"):
					for other_files in x_scripts.values():
						self.scripts.difference_update(other_files)

	def run(self):
		# group scripts by subdirectory
		split_scripts = collections.defaultdict(list)
		for f in self.scripts:
			dir_name = os.path.dirname(f[len('bin/'):])
			split_scripts[dir_name].append(f)

		base_dir = self.build_dir
		base_scripts = self.scripts
		for d, files in split_scripts.items():
			self.build_dir = os.path.join(base_dir, d)
			self.scripts = files
			self.copy_scripts()

		# restore previous values
		self.build_dir = base_dir
		self.scripts = base_scripts


class x_build_scripts_bin(x_build_scripts_custom):
	dir_name = 'bin'


class x_build_scripts_sbin(x_build_scripts_custom):
	dir_name = 'sbin'


class x_build_scripts_portagebin(x_build_scripts_custom):
	dir_name = 'portage'


class x_build_scripts(build_scripts):
	def initialize_option(self):
		build_scripts.initialize_options(self)

	def finalize_options(self):
		build_scripts.finalize_options(self)

	def run(self):
		self.run_command('build_scripts_bin')
		self.run_command('build_scripts_portagebin')
		self.run_command('build_scripts_sbin')


class x_clean(clean):
	""" clean extended for doc & post-test cleaning """

	@staticmethod
	def clean_docs():
		def get_doc_outfiles():
			for dirpath, _dirnames, filenames in os.walk('doc'):
				for f in filenames:
					if f.endswith('.docbook') or f == 'custom.xsl':
						pass
					else:
						yield os.path.join(dirpath, f)

				# do not recurse
				break


		for f in get_doc_outfiles():
			print('removing %s' % repr(f))
			os.remove(f)

		if os.path.isdir('doc/fragment'):
			remove_tree('doc/fragment')

		if os.path.isdir('doc/api/build'):
			remove_tree('doc/api/build')

	def clean_tests(self):
		# do not remove incorrect dirs accidentally
		top_dir = os.path.normpath(os.path.join(self.build_lib, '..'))
		cprefix = os.path.commonprefix((self.build_base, top_dir))
		if cprefix != self.build_base:
			return

		bin_dir = os.path.join(top_dir, 'bin')
		if os.path.exists(bin_dir):
			remove_tree(bin_dir)

		conf_dir = os.path.join(top_dir, 'cnf')
		if os.path.islink(conf_dir):
			print('removing %s symlink' % repr(conf_dir))
			os.unlink(conf_dir)

		pni_file = os.path.join(top_dir, '.portage_not_installed')
		if os.path.exists(pni_file):
			print('removing %s' % repr(pni_file))
			os.unlink(pni_file)

	def clean_man(self):
		man_dir = os.path.join(self.build_base, 'man')
		if os.path.exists(man_dir):
			remove_tree(man_dir)

	def run(self):
		if self.all:
			self.clean_tests()
			self.clean_docs()
			self.clean_man()

		clean.run(self)


class x_install(install):
	""" install command with extra Portage paths """

	user_options = install.user_options + [
		# note: $prefix and $exec_prefix are reserved for Python install
		('system-prefix=', None, "Prefix for architecture-independent data"),
		('system-exec-prefix=', None, "Prefix for architecture-specific data"),

		('bindir=', None, "Install directory for main executables"),
		('datarootdir=', None, "Data install root directory"),
		('docdir=', None, "Documentation install directory"),
		('htmldir=', None, "HTML documentation install directory"),
		('mandir=', None, "Manpage root install directory"),
		('portage-base=', 'b', "Portage install base"),
		('portage-bindir=', None, "Install directory for Portage internal-use executables"),
		('portage-datadir=', None, 'Install directory for data files'),
		('sbindir=', None, "Install directory for superuser-intended executables"),
		('sysconfdir=', None, 'System configuration path'),
	]

	# note: the order is important for proper substitution
	paths = [
		('system_prefix', '/usr'),
		('system_exec_prefix', '$system_prefix'),

		('bindir', '$system_exec_prefix/bin'),
		('sbindir', '$system_exec_prefix/sbin'),
		('sysconfdir', '/etc'),

		('datarootdir', '$system_prefix/share'),
		('docdir', '$datarootdir/doc/$package-$version'),
		('htmldir', '$docdir/html'),
		('mandir', '$datarootdir/man'),

		('portage_base', '$system_exec_prefix/lib/portage'),
		('portage_bindir', '$portage_base/bin'),
		('portage_datadir', '$datarootdir/portage'),

		# not customized at the moment
		('logrotatedir', '$sysconfdir/logrotate.d'),
		('portage_confdir', '$portage_datadir/config'),
		('portage_setsdir', '$portage_confdir/sets'),
	]

	def initialize_options(self):
		install.initialize_options(self)

		for key, default in self.paths:
			setattr(self, key, default)
		self.subst_paths = {}

	def finalize_options(self):
		install.finalize_options(self)

		# substitute variables
		new_paths = {
			'package': self.distribution.get_name(),
			'version': self.distribution.get_version(),
		}
		for key, _default in self.paths:
			new_paths[key] = subst_vars(getattr(self, key), new_paths)
			setattr(self, key, new_paths[key])
		self.subst_paths = new_paths


class x_install_data(install_data):
	""" install_data with customized path support """

	user_options = install_data.user_options

	def initialize_options(self):
		install_data.initialize_options(self)
		self.build_base = None
		self.paths = None

	def finalize_options(self):
		install_data.finalize_options(self)
		self.set_undefined_options('build',
			('build_base', 'build_base'))
		self.set_undefined_options('install',
			('subst_paths', 'paths'))

	def run(self):
		def re_sub_file(path, pattern, repl):
			print('Rewriting %s' % path)
			with codecs.open(path, 'r', 'utf-8') as f:
				data = f.read()
			data = re.sub(pattern, repl, data, flags=re.MULTILINE)
			with codecs.open(path, 'w', 'utf-8') as f:
				f.write(data)

		if create_entry_points:
			re_sub_file('cnf/repos.conf', r'= /', '= %(EPREFIX)s/')
			re_sub_file('cnf/make.globals', r'DIR="/', 'DIR="${EPREFIX}/')

		self.run_command('build_man')

		def process_data_files(df):
			for d, files in df:
				# substitute man sources
				if d.startswith('$mandir/'):
					files = [os.path.join(self.build_base, v) for v in files]

				# substitute variables in path
				d = subst_vars(d, self.paths)
				yield (d, files)

		old_data_files = self.data_files
		self.data_files = process_data_files(self.data_files)

		install_data.run(self)
		self.data_files = old_data_files


class x_install_lib(install_lib):
	""" install_lib command with Portage path substitution """

	user_options = install_lib.user_options

	def initialize_options(self):
		install_lib.initialize_options(self)
		self.portage_base = None
		self.portage_bindir = None
		self.portage_confdir = None

	def finalize_options(self):
		install_lib.finalize_options(self)
		self.set_undefined_options('install',
			('portage_base', 'portage_base'),
			('portage_bindir', 'portage_bindir'),
			('portage_confdir', 'portage_confdir'))

	def install(self):
		ret = install_lib.install(self)

		def rewrite_file(path, val_dict):
			path = os.path.join(self.install_dir, path)
			print('Rewriting %s' % path)
			with codecs.open(path, 'r', 'utf-8') as f:
				data = f.read()

			for varname, val in val_dict.items():
				regexp = r'(?m)^(%s\s*=).*$' % varname
				repl = r'\1 %s' % repr(val)

				data = re.sub(regexp, repl, data)

			with codecs.open(path, 'w', 'utf-8') as f:
				f.write(data)

		rewrite_file('portage/__init__.py', {
			'VERSION': self.distribution.get_version(),
		})

		def re_sub_file(path, pattern_repl_items):
			path = os.path.join(self.install_dir, path)
			print("Rewriting %s" % path)
			with codecs.open(path, "r", "utf-8") as f:
				data = f.read()
			for pattern, repl in pattern_repl_items:
				data = re.sub(pattern, repl, data, flags=re.MULTILINE)
			with codecs.open(path, "w", "utf-8") as f:
				f.write(data)

		val_dict = {}
		if create_entry_points:
			re_sub_file(
				"portage/const.py",
				(
					(
						r"^(GLOBAL_CONFIG_PATH\s*=\s*[\"'])(.*)([\"'])",
						lambda m: "{}{}{}".format(
							m.group(1),
							m.group(2).partition("/usr")[-1],
							m.group(3),
						),
					),
					(
						r"^(PORTAGE_BASE_PATH\s*=\s*)(.*)",
						lambda m: "{}{}".format(
							m.group(1),
							'os.path.join(os.path.realpath(__import__("sys").prefix), "lib/portage")',
						),
					),
					(
						r"^(EPREFIX\s*=\s*)(.*)",
						lambda m: "{}{}".format(
							m.group(1),
							'__import__("sys").prefix',
						),
					),
				),
			)
		else:
			val_dict.update(
				{
					"PORTAGE_BASE_PATH": self.portage_base,
					"PORTAGE_BIN_PATH": self.portage_bindir,
				}
			)
		rewrite_file("portage/const.py", val_dict)

		return ret


class x_install_scripts_custom(install_scripts):
	def initialize_options(self):
		install_scripts.initialize_options(self)
		self.root = None

	def finalize_options(self):
		self.set_undefined_options('install',
			('root', 'root'),
			(self.var_name, 'install_dir'))
		install_scripts.finalize_options(self)
		self.build_dir = os.path.join(self.build_dir, self.dir_name)

		# prepend root
		if self.root is not None:
			self.install_dir = change_root(self.root, self.install_dir)

	def run(self):
		if not create_entry_points:
			install_scripts.run(self)


class x_install_scripts_bin(x_install_scripts_custom):
	dir_name = 'bin'
	var_name = 'bindir'


class x_install_scripts_sbin(x_install_scripts_custom):
	dir_name = 'sbin'
	var_name = 'sbindir'


class x_install_scripts_portagebin(x_install_scripts_custom):
	dir_name = 'portage'
	var_name = 'portage_bindir'


class x_install_scripts(install_scripts):
	def initialize_option(self):
		pass

	def finalize_options(self):
		pass

	def run(self):
		self.run_command('install_scripts_bin')
		self.run_command('install_scripts_portagebin')
		self.run_command('install_scripts_sbin')


class x_sdist(sdist):
	""" sdist defaulting to .tar.bz2 format, and archive files owned by root """

	def initialize_options(self):
		super().initialize_options()
		self.formats = ['xztar']

	def finalize_options(self):
		if self.owner is None:
			self.owner = 'root'
		if self.group is None:
			self.group = 'root'

		sdist.finalize_options(self)


class build_tests(x_build_scripts_custom):
	""" Prepare build dir for running tests. """

	def initialize_options(self):
		x_build_scripts_custom.initialize_options(self)
		self.build_base = None
		self.build_lib = None

	def finalize_options(self):
		x_build_scripts_custom.finalize_options(self)
		self.set_undefined_options('build',
			('build_base', 'build_base'),
			('build_lib', 'build_lib'))

		# since we will be writing to $build_lib/.., it is important
		# that we do not leave $build_base
		self.top_dir = os.path.normpath(os.path.join(self.build_lib, '..'))
		cprefix = os.path.commonprefix((self.build_base, self.top_dir))
		if cprefix != self.build_base:
			raise SystemError('build_lib must be a subdirectory of build_base')

		self.build_dir = os.path.join(self.top_dir, 'bin')

	def run(self):
		self.run_command('build_py')

		# install all scripts $build_lib/../bin
		# (we can't do a symlink since we want shebangs corrected)
		x_build_scripts_custom.run(self)

		# symlink 'cnf' directory
		conf_dir = os.path.join(self.top_dir, 'cnf')
		if os.path.exists(conf_dir):
			if not os.path.islink(conf_dir):
				raise SystemError('%s exists and is not a symlink (collision)'
					% repr(conf_dir))
			os.unlink(conf_dir)
		conf_src = os.path.relpath('cnf', self.top_dir)
		print('Symlinking %s -> %s' % (conf_dir, conf_src))
		os.symlink(conf_src, conf_dir)

		# create $build_lib/../.portage_not_installed
		# to enable proper paths in tests
		with open(os.path.join(self.top_dir, '.portage_not_installed'), 'w'):
			pass


class test(Command):
	""" run tests """

	user_options = []

	def initialize_options(self):
		self.build_lib = None

	def finalize_options(self):
		self.set_undefined_options('build',
			('build_lib', 'build_lib'))

	def run(self):
		self.run_command('build_tests')
		subprocess.check_call([
			sys.executable, '-bWd',
			os.path.join(self.build_lib, 'portage/tests/runTests.py')
		])


def find_packages():
	for dirpath, _dirnames, filenames in os.walk('lib'):
		if '__init__.py' in filenames:
			yield os.path.relpath(dirpath, 'lib')


def find_scripts():
	for dirpath, _dirnames, filenames in os.walk('bin'):
		for f in filenames:
			if  f not in ['deprecated-path']:
				yield os.path.join(dirpath, f)


def get_manpages():
	linguas = os.environ.get('LINGUAS')
	if linguas is not None:
		linguas = linguas.split()

	for dirpath, _dirnames, filenames in os.walk('man'):
		groups = collections.defaultdict(list)
		for f in filenames:
			_fn, suffix = f.rsplit('.', 1)
			groups[suffix].append(os.path.join(dirpath, f))

		topdir = dirpath[len('man/'):]
		if not topdir or linguas is None or topdir in linguas:
			for g, mans in groups.items():
				yield [os.path.join('$mandir', topdir, 'man%s' % g), mans]


class build_ext(_build_ext):
	user_options = _build_ext.user_options + [
		('portage-ext-modules', None,
		 "enable portage's C/C++ extensions (cross-compiling is not supported)"),
	]

	boolean_options = _build_ext.boolean_options + [
		'portage_ext_modules',
	]

	def initialize_options(self):
		_build_ext.initialize_options(self)
		self.portage_ext_modules = None

	def run(self):
		if self.portage_ext_modules:
			_build_ext.run(self)


def venv_data_files(locations):
	if not create_entry_points:
		return
	for dest_prefix, source_path, file_args in locations:
		specific_files = []
		mode_arg = None
		for arg in file_args:
			if arg.startswith("-m"):
				mode_arg = int(arg[2:], 8)
			else:
				specific_files.append(arg)

		abs_source_path = os.path.abspath(source_path)
		for root, dirs, files in os.walk(abs_source_path):

			root_offset = root[len(abs_source_path) :].lstrip("/")
			dest_path = os.path.join(dest_prefix, root_offset)

			if specific_files:
				matched_files = list(
					itertools.chain.from_iterable(
						glob.glob(os.path.join(root, x)) for x in specific_files
					)
				)
			else:
				matched_files = [os.path.join(root, x) for x in files]

			if mode_arg:
				for filename in matched_files:
					if not os.path.islink(filename):
						os.chmod(filename, mode_arg)

			yield (dest_path, matched_files)


def get_data_files(regular_files, venv_files):
	if create_entry_points:
		return list(venv_data_files(venv_files))

	return regular_files


setup(
	name = 'portage',
	version = '3.0.22',
	url = 'https://wiki.gentoo.org/wiki/Project:Portage',
	project_urls = {
		'Release Notes': 'https://gitweb.gentoo.org/proj/portage.git/plain/RELEASE-NOTES',
		'Documentation': 'https://wiki.gentoo.org/wiki/Handbook:AMD64/Working/Portage',
	},
	author = 'Gentoo Portage Development Team',
	author_email = 'dev-portage@gentoo.org',
	description = 'Portage is the package management and distribution system for Gentoo',
	license = 'GPLV2',
	long_description = long_description,
	long_description_content_type = 'text/plain',

	package_dir = {'': 'lib'},
	packages = list(find_packages()),
	# something to cheat build & install commands
	scripts = list(find_scripts()),

	data_files = get_data_files(list(get_manpages()) + [
		['$sysconfdir', ['cnf/etc-update.conf', 'cnf/dispatch-conf.conf']],
		['$logrotatedir', ['cnf/logrotate.d/elog-save-summary']],
		['$portage_confdir', [
			'cnf/make.conf.example', 'cnf/make.globals', 'cnf/repos.conf']],
		['$portage_setsdir', ['cnf/sets/portage.conf']],
		['$docdir', ['NEWS', 'RELEASE-NOTES']],
		['$portage_base/bin', ['bin/deprecated-path']],
		['$sysconfdir/portage/repo.postsync.d', ['cnf/repo.postsync.d/example']],
	],
	[
		("etc", "cnf", ("etc-update.conf", "dispatch-conf.conf")),
		("etc/logrotate.d", "cnf/logrotate.d", ("elog-save-summary",)),
		("etc/portage/repo.postsync.d", "cnf/repo.postsync.d", ("example",)),
		(
			"share/portage/config",
			"cnf",
			("make.conf.example", "make.globals", "repos.conf"),
		),
		("share/portage/config/sets", "cnf/sets", ("*.conf",)),
		("share/man/man1", "man", ("*.1",)),
		("share/man/man5", "man", ("*.5",)),
		("share/portage/doc", "", ("NEWS", "RELEASE-NOTES")),
		("lib/portage/bin", "bin", ("-m0755",)),
	]),
	entry_points={
		"console_scripts": [
			"{}=portage.util.bin_entry_point:bin_entry_point".format(os.path.basename(path))
			for path in itertools.chain.from_iterable(x_scripts.values())
		],
	} if create_entry_points else {},
	# create_entry_points disables ext_modules, for pure python
	ext_modules = [] if create_entry_points else [
		Extension(name=n, sources=m,
		extra_compile_args=['-D_FILE_OFFSET_BITS=64',
		'-D_LARGEFILE_SOURCE', '-D_LARGEFILE64_SOURCE'])
		for n, m in x_c_helpers.items()],

	cmdclass = {
		'build': x_build,
		'build_ext': build_ext,
		'build_man': build_man,
		'build_scripts': x_build_scripts,
		'build_scripts_bin': x_build_scripts_bin,
		'build_scripts_portagebin': x_build_scripts_portagebin,
		'build_scripts_sbin': x_build_scripts_sbin,
		'build_tests': build_tests,
		'clean': x_clean,
		'docbook': docbook,
		'apidoc': apidoc,
		'install': x_install,
		'install_data': x_install_data,
		'install_docbook': install_docbook,
		'install_apidoc': install_apidoc,
		'install_lib': x_install_lib,
		'install_scripts': x_install_scripts,
		'install_scripts_bin': x_install_scripts_bin,
		'install_scripts_portagebin': x_install_scripts_portagebin,
		'install_scripts_sbin': x_install_scripts_sbin,
		'sdist': x_sdist,
		'test': test,
	},

	classifiers = [
		'Development Status :: 5 - Production/Stable',
		'Environment :: Console',
		'Intended Audience :: System Administrators',
		'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
		'Operating System :: POSIX',
		'Programming Language :: Python :: 3',
		'Topic :: System :: Installation/Setup'
	],

	python_requires = ">=3.6",
)



# test_dep_getcpv.py -- Portage Unit Testing Functionality
# Copyright 2006-2020 Gentoo Authors
# Distributed under the terms of the GNU General Public License v2

from portage.tests import TestCase
from portage.dep import dep_getcpv

class DepGetCPV(TestCase):
	""" A simple testcase for isvalidatom
	"""

	def testDepGetCPV(self):

		prefix_ops = [
			"<", ">", "=", "~", "<=",
			">=", "!=", "!<", "!>", "!~"
		]

		bad_prefix_ops = [">~", "<~", "~>", "~<"]
		postfix_ops = [("=", "*"),]

		cpvs = ["sys-apps/portage-2.1", "sys-apps/portage-2.1",
				"sys-apps/portage-2.1"]
		slots = [None, ":foo", ":2"]
		for cpv in cpvs:
			for slot in slots:
				for prefix in prefix_ops:
					mycpv = prefix + cpv
					if slot:
						mycpv += slot
					self.assertEqual(dep_getcpv(mycpv), cpv)

				for prefix, postfix in postfix_ops:
					mycpv = prefix + cpv + postfix
					if slot:
						mycpv += slot
					self.assertEqual(dep_getcpv(mycpv), cpv)



# Copyright 2017 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

import itertools


def dnf_convert(dep_struct):
	"""
	Convert dep_struct to disjunctive normal form (DNF), where dep_struct
	is either a conjunction or disjunction of the form produced by
	use_reduce(opconvert=True).
	"""
	# Normalize input to have a top-level conjunction.
	if isinstance(dep_struct, list):
		if dep_struct and dep_struct[0] == '||':
			dep_struct = [dep_struct]
	else:
		dep_struct = [dep_struct]

	conjunction = []
	disjunctions = []

	for x in dep_struct:
		if isinstance (x, list):
			assert x and x[0] == '||', \
				'Normalization error, nested conjunction found in %s' % (dep_struct,)
			if any(isinstance(element, list) for element in x):
				x_dnf = ['||']
				for element in x[1:]:
					if isinstance(element, list):
						# Due to normalization, a disjunction must not be
						# nested directly in another disjunction, so this
						# must be a conjunction.
						assert element, 'Normalization error, empty conjunction found in %s' % (x,)
						assert element[0] != '||', \
							'Normalization error, nested disjunction found in %s' % (x,)
						element = dnf_convert(element)
						if contains_disjunction(element):
							assert (len(element) == 1 and
								element[0] and element[0][0] == '||'), \
								'Normalization error, expected single disjunction in %s' % (element,)
							x_dnf.extend(element[0][1:])
						else:
							x_dnf.append(element)
					else:
						x_dnf.append(element)
				x = x_dnf
			disjunctions.append(x)
		else:
			conjunction.append(x)

	if disjunctions and (conjunction or len(disjunctions) > 1):
		dnf_form = ['||']
		for x in itertools.product(*[x[1:] for x in disjunctions]):
			normalized = conjunction[:]
			for element in x:
				if isinstance(element, list):
					normalized.extend(element)
				else:
					normalized.append(element)
			dnf_form.append(normalized)
		result = [dnf_form]
	else:
		result = conjunction + disjunctions

	return result


def contains_disjunction(dep_struct):
	"""
	Search for a disjunction contained in dep_struct, where dep_struct
	is either a conjunction or disjunction of the form produced by
	use_reduce(opconvert=True). If dep_struct is a disjunction, then
	this only returns True if there is a nested disjunction. Due to
	normalization, recursion is only needed when dep_struct is a
	disjunction containing a conjunction. If dep_struct is a conjunction,
	then it is assumed that normalization has elevated any nested
	disjunctions to the top-level.
	"""
	is_disjunction = dep_struct and dep_struct[0] == '||'
	for x in dep_struct:
		if isinstance(x, list):
			assert x, 'Normalization error, empty conjunction found in %s' % (dep_struct,)
			if x[0] == '||':
				return True
			if is_disjunction and contains_disjunction(x):
				return True
	return False



# Copyright 2013 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

from portage.tests import TestCase
from portage.tests.resolver.ResolverPlayground import (ResolverPlayground,
	ResolverPlaygroundTestCase)

class CompeteIfNewSubSlotWithoutRevBumpTestCase(TestCase):

	def testCompeteIfNewSubSlotWithoutRevBump(self):

		ebuilds = {
			"media-libs/libpng-1.5.14" : {
				"EAPI": "5",
				"SLOT": "0"
			},

			"x11-libs/gdk-pixbuf-2.26.5" : {
				"EAPI": "5",
				"DEPEND": ">=media-libs/libpng-1.4:=",
				"RDEPEND": ">=media-libs/libpng-1.4:="
			},
		}

		binpkgs = {
			"x11-libs/gdk-pixbuf-2.26.5" : {
				"EAPI": "5",
				"DEPEND": ">=media-libs/libpng-1.4:0/15=",
				"RDEPEND": ">=media-libs/libpng-1.4:0/15="
			},
		}

		installed = {
			"media-libs/libpng-1.5.14" : {
				"EAPI": "5",
				"SLOT": "0/15"
			},

			"x11-libs/gdk-pixbuf-2.26.5" : {
				"EAPI": "5",
				"DEPEND": ">=media-libs/libpng-1.4:0/15=",
				"RDEPEND": ">=media-libs/libpng-1.4:0/15="
			},
		}

		world = ["x11-libs/gdk-pixbuf"]

		test_cases = (
			# Test that --complete-graph-if-new-ver=y triggers rebuild
			# when the sub-slot changes without a revbump.
			ResolverPlaygroundTestCase(
				["media-libs/libpng"],
				options = {
					"--oneshot": True,
					"--complete-graph-if-new-ver": "y",
					"--rebuild-if-new-slot": "n",
					"--usepkg": True
				},
				success = True,
				mergelist = [
					"media-libs/libpng-1.5.14",
					"x11-libs/gdk-pixbuf-2.26.5"
				]
			),
		)

		playground = ResolverPlayground(ebuilds=ebuilds, binpkgs=binpkgs,
			installed=installed, world=world, debug=False)
		try:
			for test_case in test_cases:
				playground.run_TestCase(test_case)
				self.assertEqual(test_case.test_success, True, test_case.fail_msg)
		finally:
			playground.cleanup()



# Copyright 2018-2020 Gentoo Authors
# Distributed under the terms of the GNU General Public License v2

import errno
import os
import pty
import shutil
import socket
import tempfile

from portage.tests import TestCase
from portage.util._eventloop.global_event_loop import global_event_loop
from portage.util.futures import asyncio
from portage.util.futures.unix_events import (
	DefaultEventLoopPolicy,
	_set_nonblocking,
)


class _PipeClosedTestCase:

	def test_pipe(self):
		read_end, write_end = os.pipe()
		self._do_test(read_end, write_end)

	def test_pty_device(self):
		try:
			read_end, write_end = pty.openpty()
		except EnvironmentError:
			self.skipTest('pty not available')
		self._do_test(read_end, write_end)

	def test_domain_socket(self):
		read_end, write_end = socket.socketpair()
		self._do_test(read_end.detach(), write_end.detach())

	def test_named_pipe(self):
		tempdir = tempfile.mkdtemp()
		try:
			fifo_path = os.path.join(tempdir, 'fifo')
			os.mkfifo(fifo_path)
			self._do_test(os.open(fifo_path, os.O_NONBLOCK|os.O_RDONLY),
				os.open(fifo_path, os.O_NONBLOCK|os.O_WRONLY))
		finally:
			shutil.rmtree(tempdir)


class ReaderPipeClosedTestCase(_PipeClosedTestCase, TestCase):
	"""
	Test that a reader callback is called after the other end of
	the pipe has been closed.
	"""
	def _do_test(self, read_end, write_end):
		initial_policy = asyncio.get_event_loop_policy()
		if not isinstance(initial_policy, DefaultEventLoopPolicy):
			asyncio.set_event_loop_policy(DefaultEventLoopPolicy())

		loop = asyncio._wrap_loop()
		read_end = os.fdopen(read_end, 'rb', 0)
		write_end = os.fdopen(write_end, 'wb', 0)
		try:
			def reader_callback():
				if not reader_callback.called.done():
					reader_callback.called.set_result(None)

			reader_callback.called = loop.create_future()
			loop.add_reader(read_end.fileno(), reader_callback)

			# Allow the loop to check for IO events, and assert
			# that our future is still not done.
			loop.run_until_complete(asyncio.sleep(0, loop=loop))
			self.assertFalse(reader_callback.called.done())

			# Demonstrate that the callback is called afer the
			# other end of the pipe has been closed.
			write_end.close()
			loop.run_until_complete(reader_callback.called)
		finally:
			loop.remove_reader(read_end.fileno())
			write_end.close()
			read_end.close()
			asyncio.set_event_loop_policy(initial_policy)
			if loop not in (None, global_event_loop()):
				loop.close()
				self.assertFalse(global_event_loop().is_closed())


class WriterPipeClosedTestCase(_PipeClosedTestCase, TestCase):
	"""
	Test that a writer callback is called after the other end of
	the pipe has been closed.
	"""
	def _do_test(self, read_end, write_end):
		initial_policy = asyncio.get_event_loop_policy()
		if not isinstance(initial_policy, DefaultEventLoopPolicy):
			asyncio.set_event_loop_policy(DefaultEventLoopPolicy())

		loop = asyncio._wrap_loop()
		read_end = os.fdopen(read_end, 'rb', 0)
		write_end = os.fdopen(write_end, 'wb', 0)
		try:
			def writer_callback():
				if not writer_callback.called.done():
					writer_callback.called.set_result(None)

			writer_callback.called = loop.create_future()
			_set_nonblocking(write_end.fileno())
			loop.add_writer(write_end.fileno(), writer_callback)

			# With pypy we've seen intermittent spurious writer callbacks
			# here, so retry until the correct state is achieved.
			tries = 10
			while tries:
				tries -= 1

				# Fill up the pipe, so that no writer callbacks should be
				# received until the state has changed.
				while True:
					try:
						os.write(write_end.fileno(), 512 * b'0')
					except EnvironmentError as e:
						if e.errno != errno.EAGAIN:
							raise
						break

				# Allow the loop to check for IO events, and assert
				# that our future is still not done.
				loop.run_until_complete(asyncio.sleep(0, loop=loop))
				if writer_callback.called.done():
					writer_callback.called = loop.create_future()
				else:
					break

			self.assertFalse(writer_callback.called.done())

			# Demonstrate that the callback is called afer the
			# other end of the pipe has been closed.
			read_end.close()
			loop.run_until_complete(writer_callback.called)
		finally:
			loop.remove_writer(write_end.fileno())
			write_end.close()
			read_end.close()
			asyncio.set_event_loop_policy(initial_policy)
			if loop not in (None, global_event_loop()):
				loop.close()
				self.assertFalse(global_event_loop().is_closed())



# Copyright 2010 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

__all__ = (
	'features_set',
)

import logging

from portage.const import SUPPORTED_FEATURES
from portage.localization import _
from portage.output import colorize
from portage.util import writemsg_level

class features_set:
	"""
	Provides relevant set operations needed for access and modification of
	config.features. The FEATURES variable is automatically synchronized
	upon modification.

	Modifications result in a permanent override that will cause the change
	to propagate to the incremental stacking mechanism in config.regenerate().
	This eliminates the need to call config.backup_changes() when FEATURES
	is modified, since any overrides are guaranteed to persist despite calls
	to config.reset().
	"""

	def __init__(self, settings):
		self._settings = settings
		self._features = set()

	def __contains__(self, k):
		return k in self._features

	def __iter__(self):
		return iter(self._features)

	def _sync_env_var(self):
		self._settings['FEATURES'] = ' '.join(sorted(self._features))

	def add(self, k):
		self._settings.modifying()
		self._settings._features_overrides.append(k)
		if k not in self._features:
			self._features.add(k)
			self._sync_env_var()

	def update(self, values):
		self._settings.modifying()
		values = list(values)
		self._settings._features_overrides.extend(values)
		need_sync = False
		for k in values:
			if k in self._features:
				continue
			self._features.add(k)
			need_sync = True
		if need_sync:
			self._sync_env_var()

	def difference_update(self, values):
		self._settings.modifying()
		values = list(values)
		self._settings._features_overrides.extend('-' + k for k in values)
		remove_us = self._features.intersection(values)
		if remove_us:
			self._features.difference_update(values)
			self._sync_env_var()

	def remove(self, k):
		"""
		This never raises KeyError, since it records a permanent override
		that will prevent the given flag from ever being added again by
		incremental stacking in config.regenerate().
		"""
		self.discard(k)

	def discard(self, k):
		self._settings.modifying()
		self._settings._features_overrides.append('-' + k)
		if k in self._features:
			self._features.remove(k)
			self._sync_env_var()

	def _validate(self):
		"""
		Implements unknown-features-warn and unknown-features-filter.
		"""
		if 'unknown-features-warn' in self._features:
			unknown_features = \
				self._features.difference(SUPPORTED_FEATURES)
			if unknown_features:
				unknown_features = unknown_features.difference(
					self._settings._unknown_features)
				if unknown_features:
					self._settings._unknown_features.update(unknown_features)
					writemsg_level(colorize("BAD",
						_("FEATURES variable contains unknown value(s): %s") % \
						", ".join(sorted(unknown_features))) \
						+ "\n", level=logging.WARNING, noiselevel=-1)

		if 'unknown-features-filter' in self._features:
			unknown_features = \
				self._features.difference(SUPPORTED_FEATURES)
			if unknown_features:
				self.difference_update(unknown_features)
				self._prune_overrides()

	def _prune_overrides(self):
		"""
		If there are lots of invalid package.env FEATURES settings
		then unknown-features-filter can make _features_overrides
		grow larger and larger, so prune it. This performs incremental
		stacking with preservation of negative values since they need
		to persist for future config.regenerate() calls.
		"""
		overrides_set = set(self._settings._features_overrides)
		positive = set()
		negative = set()
		for x in self._settings._features_overrides:
			if x[:1] == '-':
				positive.discard(x[1:])
				negative.add(x[1:])
			else:
				positive.add(x)
				negative.discard(x)
		self._settings._features_overrides[:] = \
			list(positive) + list('-' + x for x in negative)



# Copyright 2011-2012 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

from itertools import chain

from portage.const import PORTAGE_PYM_PATH, PORTAGE_PYM_PACKAGES
from portage.tests import TestCase
from portage import os
from portage import _encodings
from portage import _unicode_decode

class ImportModulesTestCase(TestCase):

	def testImportModules(self):
		expected_failures = frozenset((
		))

		iters = (self._iter_modules(os.path.join(PORTAGE_PYM_PATH, x))
			for x in PORTAGE_PYM_PACKAGES)
		for mod in chain(*iters):
			try:
				__import__(mod)
			except ImportError as e:
				if mod not in expected_failures:
					self.assertTrue(False, "failed to import '%s': %s" % (mod, e))
				del e

	def _iter_modules(self, base_dir):
		for parent, dirs, files in os.walk(base_dir):
			parent = _unicode_decode(parent,
				encoding=_encodings['fs'], errors='strict')
			parent_mod = parent[len(PORTAGE_PYM_PATH)+1:]
			parent_mod = parent_mod.replace("/", ".")
			for x in files:
				x = _unicode_decode(x,
					encoding=_encodings['fs'], errors='strict')
				if x[-3:] != '.py':
					continue
				x = x[:-3]
				if x[-8:] == '__init__':
					x = parent_mod
				else:
					x = parent_mod + "." + x
				yield x



# Copyright 2010-2020 Gentoo Authors
# Distributed under the terms of the GNU General Public License v2

import locale
import logging
import time

from portage import os, _unicode_decode
from portage.exception import PortageException
from portage.localization import _
from portage.output import EOutput
from portage.util import grabfile, writemsg_level

def have_english_locale():
	lang, enc = locale.getdefaultlocale()
	if lang is not None:
		lang = lang.lower()
		lang = lang.split('_', 1)[0]
	return lang is None or lang in ('c', 'en')

def whenago(seconds):
	sec = int(seconds)
	mins = 0
	days = 0
	hrs = 0
	years = 0
	out = []

	if sec > 60:
		mins = sec // 60
		sec = sec % 60
	if mins > 60:
		hrs = mins // 60
		mins = mins % 60
	if hrs > 24:
		days = hrs // 24
		hrs = hrs % 24
	if days > 365:
		years = days // 365
		days = days % 365

	if years:
		out.append("%dy " % years)
	if days:
		out.append("%dd " % days)
	if hrs:
		out.append("%dh " % hrs)
	if mins:
		out.append("%dm " % mins)
	if sec:
		out.append("%ds " % sec)

	return "".join(out).strip()

def old_tree_timestamp_warn(portdir, settings):
	unixtime = time.time()
	default_warnsync = 30

	timestamp_file = os.path.join(portdir, "metadata/timestamp.x")
	try:
		lastsync = grabfile(timestamp_file)
	except PortageException:
		return False

	if not lastsync:
		return False

	lastsync = lastsync[0].split()
	if not lastsync:
		return False

	try:
		lastsync = int(lastsync[0])
	except ValueError:
		return False

	var_name = 'PORTAGE_SYNC_STALE'
	try:
		warnsync = float(settings.get(var_name, default_warnsync))
	except ValueError:
		writemsg_level("!!! %s contains non-numeric value: %s\n" % \
			(var_name, settings[var_name]),
			level=logging.ERROR, noiselevel=-1)
		return False

	if warnsync <= 0:
		return False

	if (unixtime - 86400 * warnsync) > lastsync:
		out = EOutput()
		if have_english_locale():
			out.ewarn("Last emerge --sync was %s ago." % \
				whenago(unixtime - lastsync))
		else:
			out.ewarn(_("Last emerge --sync was %s.") % \
				_unicode_decode(time.strftime(
				'%c', time.localtime(lastsync))))
		return True
	return False



# Copyright 2016-2020 Gentoo Authors
# Distributed under the terms of the GNU General Public License v2

__all__ = ['ConfigParserError', 'NoOptionError', 'ParsingError',
	'RawConfigParser', 'SafeConfigParser', 'read_configs']

# the following scary compatibility thing provides two classes:
# - SafeConfigParser that provides safe interpolation for values,
# - RawConfigParser that provides no interpolation for values.

import io

from configparser import (Error as ConfigParserError,
	NoOptionError, ParsingError, RawConfigParser)
from configparser import ConfigParser as SafeConfigParser

from portage import _encodings
from portage import _unicode_encode


def read_configs(parser, paths):
	"""
	Read configuration files from given paths into the specified
	ConfigParser, handling path encoding portably.
	@param parser: target *ConfigParser instance
	@type parser: SafeConfigParser or RawConfigParser
	@param paths: list of paths to read
	@type paths: iterable
	"""
	# use read_file/readfp in order to control decoding of unicode
	try:
		# Python >=3.2
		read_file = parser.read_file
		source_kwarg = 'source'
	except AttributeError:
		read_file = parser.readfp
		source_kwarg = 'filename'

	for p in paths:
		if isinstance(p, str):
			f = None
			try:
				f = io.open(_unicode_encode(p,
					encoding=_encodings['fs'], errors='strict'),
					mode='r', encoding=_encodings['repo.content'],
					errors='replace')
			except EnvironmentError:
				pass
			else:
				# The 'source' keyword argument is needed since otherwise
				# ConfigParser in Python <3.3.3 may throw a TypeError
				# because it assumes that f.name is a native string rather
				# than binary when constructing error messages.
				kwargs = {source_kwarg: p}
				read_file(f, **kwargs)
			finally:
				if f is not None:
					f.close()
		elif isinstance(p, io.StringIO):
			kwargs = {source_kwarg: "<io.StringIO>"}
			read_file(p, **kwargs)
		else:
			raise TypeError("Unsupported type %r of element %r of 'paths' argument" % (type(p), p))



# Copyright 2013-2021 Gentoo Authors
# Distributed under the terms of the GNU General Public License v2

import itertools
import logging
import stat

from portage import os
from portage.package.ebuild.fetch import DistfileName
from .DeletionTask import DeletionTask

class DeletionIterator:

	def __init__(self, config):
		self._config = config

	def __iter__(self):
		distdir = self._config.options.distfiles
		file_owners = self._config.file_owners
		whitelist = self._config.whitelist
		distfiles_local = self._config.options.distfiles_local
		deletion_db = self._config.deletion_db
		deletion_delay = self._config.options.deletion_delay
		start_time = self._config.start_time
		distfiles_set = set()
		distfiles_set.update(
			(
				filename
				if isinstance(filename, DistfileName)
				else DistfileName(filename)
				for filename in itertools.chain.from_iterable(
					layout.get_filenames(distdir) for layout in self._config.layouts
				)
			)
			if self._config.content_db is None
			else itertools.chain.from_iterable(
				(
					self._config.content_db.get_filenames_translate(filename)
					for filename in itertools.chain.from_iterable(
						layout.get_filenames(distdir) for layout in self._config.layouts
					)
				)
			)
		)
		for filename in distfiles_set:
			# require at least one successful stat()
			exceptions = []
			for layout in reversed(self._config.layouts):
				path = os.path.join(distdir, layout.get_path(filename))
				try:
					st = os.stat(path)
				except OSError as e:
					# is it a dangling symlink?
					try:
						if os.path.islink(path):
							os.unlink(path)
					except OSError as e:
						exceptions.append(e)
				else:
					if stat.S_ISREG(st.st_mode):
						break
			else:
				if exceptions:
					logging.error("stat failed on '%s' in distfiles: %s\n" %
						(filename, '; '.join(str(x) for x in exceptions)))
				continue

			if filename in file_owners:
				if deletion_db is not None:
					try:
						del deletion_db[filename]
					except KeyError:
						pass
			elif whitelist is not None and filename in whitelist:
				if deletion_db is not None:
					try:
						del deletion_db[filename]
					except KeyError:
						pass
			elif distfiles_local is not None and \
				os.path.exists(os.path.join(distfiles_local, filename)):
				if deletion_db is not None:
					try:
						del deletion_db[filename]
					except KeyError:
						pass
			else:
				self._config.scheduled_deletion_count += 1

				if deletion_db is None or deletion_delay is None:

					yield DeletionTask(background=True,
						distfile=filename,
						distfile_path=path,
						config=self._config)

				else:
					deletion_entry = deletion_db.get(filename)

					if deletion_entry is None:
						logging.debug("add '%s' to deletion db" % filename)
						deletion_db[filename] = start_time

					elif deletion_entry + deletion_delay <= start_time:

						yield DeletionTask(background=True,
							distfile=filename,
							distfile_path=path,
							config=self._config)

		if deletion_db is not None:
			for filename in list(deletion_db):
				if filename not in distfiles_set:
					try:
						del deletion_db[filename]
					except KeyError:
						pass
					else:
						logging.debug("drop '%s' from deletion db" %
							filename)
